@@ -13,6 +13,7 @@
 #include <qstring.h>
 
 #include "datablocks/unit.h"
+#include "datablocks/elementlist.h"
 
 /**
 @author Unai Garro
@@ -21,18 +22,21 @@ class Ingredient
 {
 public:
 	Ingredient();
-	Ingredient( const QString &name, double amount, const Unit &units, int unitID = -1, int ingredientID = -1, const QString &prepMethod = QString::null, int prepMethodID = -1 );
+	Ingredient( const QString &name, double amount, const Unit &units, int unitID = -1, int ingredientID = -1 );
 	~Ingredient();
+
 	int ingredientID;
 	QString name;
 	double amount;
+	double amount_offset;
 	int unitID;
 	Unit units;
-	QString prepMethod;
-	int prepMethodID;
 	int groupID;
 	QString group;
-	Ingredient & operator=( const Ingredient &i );
+
+	ElementList prepMethodList;
+
+	void setAmount( const QString &range, bool *ok = 0 );
 
 	/** Compare two elements by their id */
 	bool operator==( const Ingredient & ) const;
