@@ -16,8 +16,23 @@
 #include <qdatetime.h>
 
 #include "ingredientlist.h"
+#include "datablocks/rating.h"
 #include "datablocks/elementlist.h"
 
+class Yield
+{
+public:
+	Yield() : amount(1), amount_offset(0), type(QString::null), type_id(-1){}
+
+	QString amountToString() const;
+	QString toString() const;
+
+	double amount;
+	double amount_offset;
+	QString type;
+	int type_id;
+};
+
 /**
 @author Unai Garro
 */
@@ -29,7 +44,7 @@ public:
 	// Public variables
 
 	int recipeID;
-	int persons;
+	Yield yield;
 	QString title;
 	QString instructions;
 	QPixmap photo;
@@ -38,6 +53,12 @@ public:
 	ElementList authorList; //authors' id+name
 	QTime prepTime;
 
+	QDateTime ctime;
+	QDateTime mtime;
+	QDateTime atime;
+
+	RatingList ratingList;
+
 	// Public methods
 	void empty( void );
 
