@@ -24,6 +24,7 @@
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 #include <qwidgetstack.h>
+#include <qpainter.h>
 
 #include <kapplication.h>
 #include <kcompletionbox.h>
@@ -37,6 +38,8 @@
 #include "createunitdialog.h"
 #include "selectauthorsdialog.h"
 #include "resizerecipedialog.h"
+#include "ingredientparserdialog.h"
+#include "editratingdialog.h"
 #include "datablocks/recipe.h"
 #include "datablocks/categorytree.h"
 #include "datablocks/unit.h"
@@ -47,8 +50,12 @@
 #include "widgets/ingredientcombobox.h"
 #include "widgets/headercombobox.h"
 #include "widgets/prepmethodcombobox.h"
+#include "widgets/inglistviewitem.h"
+#include "../widgets/ratingdisplaywidget.h"
+#include "widgets/kwidgetlistbox.h"
 #include "image.h" //Initializes default photo
 
+#include "profiling.h"
 
 ImageDropLabel::ImageDropLabel( QWidget *parent, QPixmap &_sourcePhoto ) : QLabel( parent ),
 		sourcePhoto( _sourcePhoto )
@@ -82,168 +89,6 @@ void ImageDropLabel::dropEvent( QDropEvent* event )
 	}
 }
 
-#define INGLISTVIEWITEM_RTTI 1004
-#define INGGRPLISTVIEWITEM_RTTI 1003
-
-class IngListViewItem : public QListViewItem
-{
-public:
-	IngListViewItem( QListView* qlv, const Ingredient &i ) : QListViewItem( qlv )
-	{
-		init( i );
-	}
-
-	IngListViewItem( QListView* qlv, QListViewItem *after, const Ingredient &i ) : QListViewItem( qlv, after )
-	{
-		init( i );
-	}
-
-	IngListViewItem( QListViewItem* qli, QListViewItem *after, const Ingredient &i ) : QListViewItem( qli, after )
-	{
-		init( i );
-	}
-
-	int rtti() const
-	{
-		return INGLISTVIEWITEM_RTTI;
-	}
-
-	Ingredient ingredient() const
-	{
-		return m_ing;
-	}
-
-	void setAmount( double amount )
-	{
-		amount_str = QString::null;
-
-		if ( amount > 0 ) {
-			KConfig * config = kapp->config();
-			config->setGroup( "Formatting" );
-
-			if ( config->readBoolEntry( "Fraction" ) )
-				amount_str = MixedNumber( amount ).toString();
-			else
-				amount_str = beautify( KGlobal::locale() ->formatNumber( amount, 5 ) );
-		}
-
-		m_ing.amount = amount;
-
-		//FIXME: make sure the right unit is showing after changing this (force a repaint... repaint() doesn't do the job right because it gets caught in a loop)
-	}
-
-	void setUnit( const Unit &unit )
-	{
-		//### This shouldn't be necessary... the db backend should ensure this doesn't happen
-		if ( !unit.name.isEmpty() )
-			m_ing.units.name = unit.name;
-		if ( !unit.plural.isEmpty() )
-			m_ing.units.plural = unit.plural;
-	}
-
-	void setPrepMethod( const QString &prepMethod )
-	{
-		m_ing.prepMethod = prepMethod;
-	}
-
-	virtual void setText( int column, const QString &text )
-	{
-		switch ( column ) {
-		case 0:
-			break;
-		case 1:
-			setAmount( MixedNumber::fromString( text ).toDouble() );
-			break;
-		case 2:
-			setUnit( Unit( text, m_ing.amount ) );
-			break;
-		case 3:
-			setPrepMethod( text );
-			break;
-		default:
-			break;
-		}
-	}
-
-protected:
-	Ingredient m_ing;
-	QString amount_str;
-
-public:
-	virtual QString text( int column ) const
-	{
-		switch ( column ) {
-		case 0:
-			return m_ing.name;
-			break;
-		case 1:
-			return amount_str;
-			break;
-		case 2:
-			return ( m_ing.amount > 1 ) ? m_ing.units.plural : m_ing.units.name;
-			break;
-		case 3:
-			return m_ing.prepMethod;
-			break;
-		default:
-			return ( QString::null );
-		}
-	}
-
-private:
-	void init( const Ingredient &i )
-	{
-		m_ing = i;
-
-		setAmount( i.amount );
-	}
-};
-
-class IngGrpListViewItem : public QListViewItem
-{
-public:
-	IngGrpListViewItem( QListView* qlv, QListViewItem *after, const QString &group, int id ) : QListViewItem( qlv, after )
-	{
-		init( group, id );
-	}
-
-	int rtti() const
-	{
-		return INGGRPLISTVIEWITEM_RTTI;
-	}
-
-	QString group() const
-	{
-		return m_group;
-	}
-	int id() const
-	{
-		return m_id;
-	}
-
-	virtual QString text( int column ) const
-	{
-		switch ( column ) {
-		case 0:
-			return m_group + ":";
-			break;
-		default:
-			return ( QString::null );
-		}
-	}
-
-protected:
-	QString m_group;
-	int m_id;
-
-private:
-	void init( const QString &group, int id )
-	{
-		m_group = group;
-		m_id = id;
-	}
-};
-
 
 RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( parent )
 {
@@ -260,18 +105,8 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	// Tabs
 	tabWidget = new QTabWidget( this, "tabWidget" );
 	tabWidget->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
-	//tabWidget->setAutoMask( TRUE );
 
 
-
-	// Spacers to ingredients and instructions
-	//QSpacerItem* spacer_bottomphoto = new QSpacerItem( 10,10, QSizePolicy::Minimum, QSizePolicy::Fixed );
-	//layout->addItem( spacer_bottomphoto,6,1);
-	//QSpacerItem* spacerToIngredients = new QSpacerItem( 10,200, QSizePolicy::Minimum, QSizePolicy::Fixed );
-	//layout->addMultiCell( spacerToIngredients, 1,4,3,3 );
-	//QSpacerItem* spacer_rightGBox = new QSpacerItem( 10,10, QSizePolicy::Fixed, QSizePolicy::MinimumExpanding );
-	//layout->addItem( spacer_rightGBox,7,4 );
-
 	//------- Recipe Tab -----------------
 	// Recipe Photo
 
@@ -381,18 +216,22 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	addCategoryButton->setFlat( true );
 
 	//Category ->Servings spacer
-	QSpacerItem* category_servings = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::Fixed );
-	recipeLayout->addItem( category_servings, 5, 4 );
+	QSpacerItem* category_yield = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::Fixed );
+	recipeLayout->addItem( category_yield, 5, 4 );
 
 	QHBox *serv_prep_box = new QHBox( recipeTab );
+	serv_prep_box->setSpacing( 5 );
 
-	QVBox *servingsBox = new QVBox( serv_prep_box );
-	servingsBox->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
-	servingsBox->setSpacing( 5 );
+	// Backup options
+	QGroupBox *yieldGBox = new QGroupBox( serv_prep_box, "yieldGBox" );
+	yieldGBox->setTitle( i18n( "Yield" ) );
+	yieldGBox->setColumns( 2 );
 
-	servingsLabel = new QLabel( i18n( "Servings" ), servingsBox );
-	servingsNumInput = new KIntNumInput( servingsBox );
-	servingsNumInput->setMinValue( 1 );
+	yieldLabel = new QLabel( i18n( "Amount" ), yieldGBox );
+	QLabel *yieldTypeLabel = new QLabel( i18n( "Type" ), yieldGBox );
+	yieldNumInput = new FractionInput( yieldGBox );
+	yieldNumInput->setAllowRange(true);
+	yieldTypeEdit = new KLineEdit( yieldGBox );
 
 	QVBox *prepTimeBox = new QVBox( serv_prep_box );
 	prepTimeBox->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
@@ -449,6 +288,7 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	QVBox *amountVBox = new QVBox( allInputHBox );
 	amountLabel = new QLabel( i18n( "Amount:" ), amountVBox );
 	amountEdit = new FractionInput( amountVBox );
+	amountEdit->setAllowRange(true);
 	amountEdit->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
 
 	QVBox *unitVBox = new QVBox( allInputHBox );
@@ -516,6 +356,14 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	removeButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
 	ingredientsLayout->addWidget( removeButton, 7, 5 );
 
+	ingParserButton = new KPushButton( ingredientGBox );
+	ingParserButton->setFixedSize( QSize( 31, 31 ) );
+	ingParserButton->setFlat( true );
+	pm = il->loadIcon( "editpaste", KIcon::NoGroup, 16 );
+	ingParserButton->setPixmap( pm );
+	ingParserButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+	ingredientsLayout->addWidget( ingParserButton, 8, 5 );
+
 	QToolTip::add
 		( addButton, i18n( "Add ingredient" ) );
 	QToolTip::add
@@ -524,6 +372,8 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 		( downButton, i18n( "Move ingredient down" ) );
 	QToolTip::add
 		( removeButton, i18n( "Remove ingredient" ) );
+	QToolTip::add
+		( ingParserButton, i18n( "Paste Ingredients" ) );
 
 	// Ingredient List
 	ingredientList = new KListView( ingredientGBox, "ingredientList" );
@@ -540,7 +390,7 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	ingredientList->setRenameable( 2, true ); //units
 	ingredientList->setRenameable( 3, true ); //prep method
 	ingredientList->setDefaultRenameAction( QListView::Reject );
-	ingredientsLayout->addMultiCellWidget( ingredientList, 3, 8, 1, 3 );
+	ingredientsLayout->addMultiCellWidget( ingredientList, 3, 9, 1, 3 );
 
 	// ------- Recipe Instructions Tab -----------
 
@@ -563,9 +413,22 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 
 	// ------- END OF Recipe Instructions Tab -----------
 
+
+	// ------- Recipe Ratings Tab -----------
+
+	QVBox *ratingsTab = new QVBox(recipeTab);
+	ratingListDisplayWidget = new KWidgetListbox(ratingsTab);
+	QPushButton *addRatingButton = new QPushButton(i18n("Add Rating..."),ratingsTab);
+
+	connect( addRatingButton, SIGNAL(clicked()), this, SLOT(slotAddRating()) );
+
+	// ------- END OF Recipe Ratings Tab -----------
+
+
 	tabWidget->insertTab( recipeTab, i18n( "Recipe" ) );
 	tabWidget->insertTab( ingredientGBox, i18n( "Ingredients" ) );
 	tabWidget->insertTab( instructionsTab, i18n( "Instructions" ) );
+	tabWidget->insertTab( ratingsTab, i18n( "Ratings" ) );
 
 
 	// Functions Box
@@ -612,9 +475,11 @@ RecipeInputDialog::RecipeInputDialog( QWidget* parent, RecipeDB *db ) : QVBox( p
 	connect( downButton, SIGNAL( clicked() ), this, SLOT( moveIngredientDown() ) );
 	connect( removeButton, SIGNAL( clicked() ), this, SLOT( removeIngredient() ) );
 	connect( addButton, SIGNAL( clicked() ), this, SLOT( addIngredient() ) );
+	connect( ingParserButton, SIGNAL( clicked() ), this, SLOT( slotIngredientParser() ) );
 	connect( photoLabel, SIGNAL( changed() ), this, SIGNAL( changed() ) );
 	connect( this, SIGNAL( changed() ), this, SLOT( recipeChanged() ) );
-	connect( servingsNumInput, SIGNAL( valueChanged( int ) ), this, SLOT( recipeChanged() ) );
+	connect( yieldNumInput, SIGNAL( textChanged( const QString & ) ), this, SLOT( recipeChanged() ) );
+	connect( yieldTypeEdit, SIGNAL( textChanged( const QString & ) ), this, SLOT( recipeChanged() ) );
 	connect( prepTimeEdit, SIGNAL( valueChanged( const QTime & ) ), SLOT( recipeChanged() ) );
 	connect( titleEdit, SIGNAL( textChanged( const QString& ) ), this, SLOT( recipeChanged( const QString& ) ) );
 	connect( instructionsEdit, SIGNAL( textChanged() ), this, SLOT( recipeChanged() ) );
@@ -692,8 +557,7 @@ void RecipeInputDialog::loadRecipe( int recipeID )
 	tabWidget->setCurrentPage( 0 );
 
 	// Load specified Recipe ID
-	database->loadRecipe( loadedRecipe, RecipeDB::All, recipeID );
-
+	database->loadRecipe( loadedRecipe, RecipeDB::All ^ RecipeDB::Meta, recipeID );
 
 	reload();
 
@@ -709,18 +573,21 @@ void RecipeInputDialog::reload( void )
 
 	unitComboList->clear();
 	reloadCombos();
-	servingsNumInput->setValue( 1 );
+	yieldNumInput->setValue( 1, 0 );
+	yieldTypeEdit->setText("");
 	amountEdit->clear();
 	ingredientList->clear();
 	ingredientBox->lineEdit()->setText("");
 	prepMethodBox->lineEdit()->setText("");
 	headerBox->lineEdit()->setText("");
 	unitBox->lineEdit()->setText("");
+	ratingListDisplayWidget->clear();
 
 	//Load Values in Interface
 	titleEdit->setText( loadedRecipe->title );
 	instructionsEdit->setText( loadedRecipe->instructions );
-	servingsNumInput->setValue( loadedRecipe->persons );
+	yieldNumInput->setValue( loadedRecipe->yield.amount, loadedRecipe->yield.amount_offset );
+	yieldTypeEdit->setText( loadedRecipe->yield.type );
 	prepTimeEdit->setTime( loadedRecipe->prepTime );
 
 	//show ingredient list
@@ -775,7 +642,9 @@ void RecipeInputDialog::reload( void )
 		}
 	}
 	else {
-		photoLabel->setPixmap( QPixmap( defaultPhoto ) );
+		QPixmap photo = QPixmap( defaultPhoto );
+		photoLabel->setPixmap( photo );
+		sourcePhoto.resize( 0, 0 );
 	}
 
 
@@ -784,6 +653,15 @@ void RecipeInputDialog::reload( void )
 
 	// Show authors
 	showAuthors();
+
+	// Show ratings
+	for ( RatingList::iterator rating_it = loadedRecipe->ratingList.begin(); rating_it != loadedRecipe->ratingList.end(); ++rating_it ) {
+		RatingDisplayWidget *item = new RatingDisplayWidget;
+		item->rating_it = rating_it;
+		addRating(*rating_it,item);
+		ratingListDisplayWidget->insertItem(item);
+	}
+	ratingListDisplayWidget->ensureCellVisible(0,0);
 }
 
 void RecipeInputDialog::loadUnitListCombo( void )
@@ -1067,29 +945,61 @@ int RecipeInputDialog::createNewUnitIfNecessary( const QString &unit, bool plura
 	}
 }
 
-int RecipeInputDialog::createNewPrepIfNecessary( const QString &prep )
+QValueList<int> RecipeInputDialog::createNewPrepIfNecessary( const QString &prep )
 {
-	if ( prep.stripWhiteSpace().isEmpty() )  //no prep method
-		return -1;
-	else if ( !prepMethodBox->contains( prep ) )  //creating new
+	QValueList<int> ids;
+
+	if ( prep.stripWhiteSpace().isEmpty() )  //no prep methods
+		return ids;
+	else
 	{
-		int id = database->findExistingPrepByName( prep );
-		if ( -1 == id )
+		QStringList prepMethods = QStringList::split(",",prep);
+		for ( QStringList::const_iterator it = prepMethods.begin(); it != prepMethods.end(); ++it ) {
+			int id = database->findExistingPrepByName( (*it).stripWhiteSpace() );
+			if ( id == -1 )
+			{
+				database->createNewPrepMethod( (*it).stripWhiteSpace() );
+				id = database->lastInsertID();
+			}
+			ids << id;
+		}
+
+		return ids;
+	}
+}
+
+int RecipeInputDialog::createNewGroupIfNecessary( const QString &group )
+{
+	if ( group.stripWhiteSpace().isEmpty() )  //no group
+		return -1;
+	else
+	{
+		int id = database->findExistingIngredientGroupByName( group );
+		if ( id == -1 ) //creating new
 		{
-			database->createNewPrepMethod( prep );
+			database->createNewIngGroup( group );
 			id = database->lastInsertID();
 		}
 
 		return id;
 	}
-	else //already exists
-		return prepMethodBox->id( prep );
 }
 
-int RecipeInputDialog::createNewGroupIfNecessary( const QString &group )
+int RecipeInputDialog::createNewYieldIfNecessary( const QString &yield )
 {
-	database->createNewIngGroup( group );
-	return database->lastInsertID();
+	if ( yield.stripWhiteSpace().isEmpty() )  //no yield
+		return -1;
+	else
+	{
+		int id = database->findExistingYieldTypeByName( yield );
+		if ( id == -1 ) //creating new
+		{
+			database->createNewYieldType( yield );
+			id = database->lastInsertID();
+		}
+
+		return id;
+	}
 }
 
 bool RecipeInputDialog::checkAmountEdit()
@@ -1121,11 +1031,15 @@ bool RecipeInputDialog::checkBounds()
 		return false;
 	}
 
-	if ( prepMethodBox->currentText().length() > database->maxPrepMethodNameLength() ) {
-		KMessageBox::error( this, QString( i18n( "Preparation method cannot be longer than %1 characters." ) ).arg( database->maxPrepMethodNameLength() ) );
-		prepMethodBox->lineEdit() ->setFocus();
-		prepMethodBox->lineEdit() ->selectAll();
-		return false;
+	QStringList prepMethodList = QStringList::split(",",prepMethodBox->currentText());
+	for ( QStringList::const_iterator it = prepMethodList.begin(); it != prepMethodList.end(); ++it ) {
+		if ( (*it).stripWhiteSpace().length() > database->maxPrepMethodNameLength() )
+		{
+			KMessageBox::error( this, QString( i18n( "Preparation method cannot be longer than %1 characters." ) ).arg( database->maxPrepMethodNameLength() ) );
+			prepMethodBox->lineEdit() ->setFocus();
+			prepMethodBox->lineEdit() ->selectAll();
+			return false;
+		}
 	}
 
 	return true;
@@ -1161,10 +1075,10 @@ void RecipeInputDialog::addIngredient( void )
 		createNewIngredientIfNecessary();
 
 		Unit new_unit;
-		int unitID = createNewUnitIfNecessary( unit_text, ( amountEdit->value() > 1 ) ? true : false, ingredientBox->currentText().stripWhiteSpace(), new_unit );
+		int unitID = createNewUnitIfNecessary( unit_text, ( amountEdit->maxValue() > 1 ) ? true : false, ingredientBox->currentText().stripWhiteSpace(), new_unit );
 		if ( unitID == -1 )  // this will happen if the dialog to create a unit was cancelled
 			return ;
-		int prepID = createNewPrepIfNecessary( prepMethodBox->currentText() );
+		QValueList<int> prepIDs = createNewPrepIfNecessary( prepMethodBox->currentText() );
 
 		//Add it first to the Recipe list then to the ListView
 		if ( ( ingredientBox->count() > 0 ) && ( unitBox->count() > 0 ) )
@@ -1172,13 +1086,18 @@ void RecipeInputDialog::addIngredient( void )
 			Ingredient ing;
 
 			ing.name = ingredientBox->currentText();
-			ing.amount = amountEdit->value().toDouble();
+
+			amountEdit->value(ing.amount,ing.amount_offset);
 			ing.units = new_unit;
 			ing.unitID = unitID;
 			ing.ingredientID = ingredientBox->id( ingredientBox->currentItem() );
 
-			ing.prepMethod = prepMethodBox->currentText();
-			ing.prepMethodID = prepID;
+			ing.prepMethodList = ElementList::split(",",prepMethodBox->currentText());
+
+			QValueList<int>::const_iterator id_it = prepIDs.begin();
+			for ( ElementList::iterator it = ing.prepMethodList.begin(); it != ing.prepMethodList.end(); ++it, ++id_it ) {
+				(*it).id = *id_it;
+			}
 
 			//Append also to the ListView
 			QListViewItem* lastElement = ingredientList->lastItem();
@@ -1202,8 +1121,12 @@ void RecipeInputDialog::addIngredient( void )
 		}
 	}
 
+	ingredientBox->lineEdit()->clear();
+	amountEdit->clear();
+	unitBox->lineEdit()->clear();
+	prepMethodBox->lineEdit()->clear();
+
 	ingredientBox->setFocus(); //put cursor back to the ingredient name so user can begin next ingredient
-	ingredientBox->lineEdit() ->selectAll();
 }
 
 void RecipeInputDialog::syncListView( QListViewItem* it, const QString &new_text, int col )
@@ -1220,24 +1143,30 @@ void RecipeInputDialog::syncListView( QListViewItem* it, const QString &new_text
 	case 1:  //amount
 		{
 			bool ok;
-			MixedNumber new_mn = MixedNumber::fromString( new_text, &ok );
-			MixedNumber prev_mn( ( *ing ).amount );
+			
+			Ingredient new_ing_amount;
+			new_ing_amount.setAmount(new_text,&ok);
+
 			if ( ok )
 			{
-				( *ing ).amount = new_mn.toDouble();
-				if ( !new_text.isEmpty() )
-					ing_item->setAmount( new_mn.toDouble() );
+				if ( (*ing).amount != new_ing_amount.amount ||
+				     ( *ing ).amount_offset != new_ing_amount.amount_offset ) {
+					( *ing ).amount = new_ing_amount.amount;
+					( *ing ).amount_offset = new_ing_amount.amount_offset;
+					if ( !new_text.isEmpty() )
+						ing_item->setAmount( new_ing_amount.amount, new_ing_amount.amount_offset );
+
+					( *ing ).amount = new_ing_amount.amount;
+					( *ing ).amount_offset = new_ing_amount.amount_offset;
+					emit changed();
+				}
 			}
 			else
 			{
 				if ( !new_text.isEmpty() )
-					ing_item->setAmount( prev_mn.toDouble() );
+ 					ing_item->setAmount( ( *ing ).amount, ( *ing ).amount_offset );
 			}
 
-			if ( new_mn != prev_mn )
-			{
-				emit changed();
-			}
 			break;
 		}
 	case 2:  //unit
@@ -1273,21 +1202,29 @@ void RecipeInputDialog::syncListView( QListViewItem* it, const QString &new_text
 		}
 	case 3:  //prep method
 		{
-			QString old_text = ( *ing ).prepMethod;
+			QString old_text = ( *ing ).prepMethodList.join(",");
 
-			if ( new_text.length() > database->maxPrepMethodNameLength() )
-			{
-				KMessageBox::error( this, QString( i18n( "Preparation method cannot be longer than %1 characters." ) ).arg( database->maxPrepMethodNameLength() ) );
-				ing_item->setPrepMethod( old_text );
-				break;
+			QStringList prepMethodList = QStringList::split(",",new_text.stripWhiteSpace());
+
+			for ( QStringList::const_iterator it = prepMethodList.begin(); it != prepMethodList.end(); ++it ) {
+				if ( (*it).stripWhiteSpace().length() > database->maxPrepMethodNameLength() )
+				{
+					KMessageBox::error( this, QString( i18n( "Preparation method cannot be longer than %1 characters." ) ).arg( database->maxPrepMethodNameLength() ) );
+					ing_item->setPrepMethod( old_text );
+					break;
+				}
 			}
 
 			if ( old_text != new_text.stripWhiteSpace() )
 			{
-				int new_id = createNewPrepIfNecessary( new_text.stripWhiteSpace() );
+				QValueList<int> new_ids = createNewPrepIfNecessary( new_text.stripWhiteSpace() );
 
-				( *ing ).prepMethod = new_text;
-				( *ing ).prepMethodID = new_id;
+				(*ing).prepMethodList = ElementList::split(",",new_text.stripWhiteSpace());
+	
+				QValueList<int>::const_iterator id_it = new_ids.begin();
+				for ( ElementList::iterator it = (*ing).prepMethodList.begin(); it != (*ing).prepMethodList.end(); ++it, ++id_it ) {
+					(*it).id = *id_it;
+				}
 
 				emit changed();
 			}
@@ -1341,7 +1278,8 @@ void RecipeInputDialog::saveRecipe( void )
 	loadedRecipe->photo = sourcePhoto;
 	loadedRecipe->instructions = instructionsEdit->text();
 	loadedRecipe->title = titleEdit->text();
-	loadedRecipe->persons = servingsNumInput->value();
+	yieldNumInput->value(loadedRecipe->yield.amount,loadedRecipe->yield.amount_offset);
+	loadedRecipe->yield.type_id = createNewYieldIfNecessary(yieldTypeEdit->text());
 	loadedRecipe->prepTime = prepTimeEdit->time();
 
 	// Now save()
@@ -1361,11 +1299,12 @@ void RecipeInputDialog::newRecipe( void )
 	instructionsEdit->setText( i18n( "Write the recipe instructions here" ) );
 	instructionsEdit->clearCompletionItems();
 	titleEdit->setText( i18n( "Write the recipe title here" ) );
-	amountEdit->setValue( 0.0 );
+	amountEdit->setValue( 0.0, 0.0 );
 	ingredientList->clear();
 	authorShow->clear();
 	categoryShow->clear();
-	servingsNumInput->setValue( 1 );
+	yieldNumInput->setValue( 1, 0 );
+	yieldTypeEdit->setText("");
 	prepTimeEdit->setTime( QTime( 0, 0 ) );
 
 	instructionsEdit->selectAll();
@@ -1384,13 +1323,22 @@ void RecipeInputDialog::newRecipe( void )
 
 void RecipeInputDialog::reloadCombos( void )  //Reloads lists of units and preparation methods
 {
-	//these only needed to be reloaded once
-	if ( ingredientBox->count() == 0 )
+	//these only needed to be loaded once
+	if ( ingredientBox->count() == 0 ) {
+		START_TIMER("Loading ingredient input auto-completion");
 		ingredientBox->reload();
-	if ( headerBox->count() == 0 )
+		END_TIMER();
+	}
+	if ( headerBox->count() == 0 ) {
+		START_TIMER("Loading ingredient header input auto-completion");
 		headerBox->reload();
-	if ( prepMethodBox->count() == 0 )
+		END_TIMER();
+	}
+	if ( prepMethodBox->count() == 0 ) {
+		START_TIMER("Loading prep method input auto-completion");
 		prepMethodBox->reload();
+		END_TIMER();
+	}
 
 	loadUnitListCombo();
 }
@@ -1559,7 +1507,7 @@ void RecipeInputDialog::spellCheck( void )
 
 void RecipeInputDialog::resizeRecipe( void )
 {
-	loadedRecipe->persons = servingsNumInput->value();
+	yieldNumInput->value( loadedRecipe->yield.amount, loadedRecipe->yield.amount_offset );
 	ResizeRecipeDialog dlg( this, loadedRecipe );
 
 	if ( dlg.exec() == QDialog::Accepted )
@@ -1605,4 +1553,145 @@ void RecipeInputDialog::typeButtonClicked( int button_id )
 	}
 }
 
+//TODO: This inputs the ingredients by simulating the user's actions that would do this.
+//	Instead, this shouldn't require messing with the UI.
+void RecipeInputDialog::slotIngredientParser()
+{
+	UnitList units;
+	database->loadUnits(&units);
+	IngredientParserDialog dlg(units,this);
+	if ( dlg.exec() == QDialog::Accepted ) {
+		IngredientList ings = dlg.ingredients();
+		QStringList usedGroups;
+		for ( IngredientList::iterator it = ings.begin(); it != ings.end(); ++it ) {
+			if ( !(*it).group.isEmpty() && usedGroups.find((*it).group) == usedGroups.end() ) {
+				typeButtonGrp->setButton( 1 );
+				headerBox->lineEdit()->setText((*it).group);
+				addIngredient(); //adds the header
+				usedGroups << (*it).group;
+			}
+
+			ingredientBox->lineEdit()->setText((*it).name.left( database->maxIngredientNameLength()));
+			amountEdit->setValue((*it).amount,(*it).amount_offset);
+			unitBox->lineEdit()->setText((*it).units.name.left( database->maxUnitNameLength()));
+
+			for ( ElementList::iterator prep_it = (*it).prepMethodList.begin(); prep_it != (*it).prepMethodList.end(); ++prep_it ) {
+				(*prep_it).name = (*prep_it).name.left(database->maxPrepMethodNameLength());
+			}
+			prepMethodBox->lineEdit()->setText((*it).prepMethodList.join(","));
+
+			addIngredient();
+
+			if ( usedGroups.count() > 0 && (*it).group.isEmpty() ) {
+				QListViewItem *last_item = ingredientList->lastItem();
+				if ( last_item->parent() ) {
+					last_item->parent()->takeItem( last_item );
+					ingredientList->insertItem( last_item );
+					last_item->moveItem( ingredientList->lastItem()->parent() );
+				}
+			}
+		}
+	}
+}
+
+void RecipeInputDialog::slotAddRating()
+{
+	ElementList criteriaList;
+	database->loadRatingCriterion(&criteriaList);
+
+	EditRatingDialog ratingDlg(criteriaList,this);
+	if ( ratingDlg.exec() == QDialog::Accepted ) {
+		Rating r = ratingDlg.rating();
+
+		for ( RatingCriteriaList::iterator rc_it = r.ratingCriteriaList.begin(); rc_it != r.ratingCriteriaList.end(); ++rc_it ) {
+			int criteria_id = database->findExistingRatingByName((*rc_it).name);
+			if ( criteria_id == -1 ) {
+				database->createNewRating((*rc_it).name);
+				criteria_id = database->lastInsertID();
+			}
+			(*rc_it).id = criteria_id;
+		}
+
+		RatingDisplayWidget *item = new RatingDisplayWidget;
+		item->rating_it = loadedRecipe->ratingList.append(r);
+		addRating(r,item);
+		ratingListDisplayWidget->insertItem(item,0);
+		emit( recipeChanged() ); //Indicate that the recipe changed
+	}
+}
+
+void RecipeInputDialog::addRating( const Rating &rating, RatingDisplayWidget *item )
+{
+	int average = qRound(rating.average());
+	if ( average >= 0 )
+		item->icon->setPixmap( UserIcon(QString("rating%1").arg(average) ) );
+	else //no rating criteria, therefore no average (we don't want to automatically assume a zero average)
+		item->icon->clear();
+
+	item->raterName->setText(rating.rater);
+	item->comment->setText(rating.comment);
+
+	item->criteriaListView->clear();
+	for ( RatingCriteriaList::const_iterator rc_it = rating.ratingCriteriaList.begin(); rc_it != rating.ratingCriteriaList.end(); ++rc_it ) {
+		QListViewItem * it = new QListViewItem(item->criteriaListView,(*rc_it).name);
+	
+		int stars = int((*rc_it).stars * 2); //multiply by two to make it easier to work with half-stars
+	
+		QPixmap star = UserIcon(QString::fromLatin1("star_on"));
+		int pixmapWidth = 18*(stars/2)+((stars%2==1)?9:0);
+		QPixmap generatedPixmap(pixmapWidth,18);
+	
+		if ( !generatedPixmap.isNull() ) { //there aren't zero stars
+			generatedPixmap.fill();
+			QPainter painter( &generatedPixmap );
+			painter.drawTiledPixmap(0,0,pixmapWidth,18,star);
+			it->setPixmap(1,generatedPixmap);
+		}
+	}
+
+	item->buttonEdit->disconnect();
+	item->buttonRemove->disconnect();
+	connect(item->buttonEdit, SIGNAL(clicked()),
+		this, SLOT(slotEditRating()));
+	connect(item->buttonRemove, SIGNAL(clicked()),
+		this, SLOT(slotRemoveRating()));
+}
+
+void RecipeInputDialog::slotEditRating()
+{
+	RatingDisplayWidget *sender = (RatingDisplayWidget*)(QObject::sender()->parent());
+
+	ElementList criteriaList;
+	database->loadRatingCriterion(&criteriaList);
+
+	EditRatingDialog ratingDlg(criteriaList,*sender->rating_it,this);
+	if ( ratingDlg.exec() == QDialog::Accepted ) {
+		Rating r = ratingDlg.rating();
+
+		for ( RatingCriteriaList::iterator rc_it = r.ratingCriteriaList.begin(); rc_it != r.ratingCriteriaList.end(); ++rc_it ) {
+			int criteria_id = database->findExistingRatingByName((*rc_it).name);
+			if ( criteria_id == -1 ) {
+				database->createNewRating((*rc_it).name);
+				criteria_id = database->lastInsertID();
+			}
+			(*rc_it).id = criteria_id;
+		}
+
+		(*sender->rating_it) = r;
+		addRating(r,sender);
+		emit recipeChanged(); //Indicate that the recipe changed
+	}
+}
+
+void RecipeInputDialog::slotRemoveRating()
+{
+	RatingDisplayWidget *sender = (RatingDisplayWidget*)(QObject::sender()->parent());
+	loadedRecipe->ratingList.remove(sender->rating_it);
+
+	//FIXME: sender is removed but never deleted (sender->deleteLater() doesn't work)
+	ratingListDisplayWidget->removeItem(sender);
+
+	emit recipeChanged(); //Indicate that the recipe changed
+}
+
 #include "recipeinputdialog.moc"
