@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2003-2004                                               *
+*   Copyright (C) 2003-2005                                               *
 *   Jason Kivlighn (mizunoami44@users.sourceforge.net)                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -17,6 +17,8 @@
 #include <kcolordialog.h>
 #include <klocale.h>
 #include <kpopupmenu.h>
+#include <kruler.h>
+#include <kiconloader.h>
 
 #include <qaction.h>
 #include <qlabel.h>
@@ -26,33 +28,64 @@
 #include <qtooltip.h>
 #include <qobjectlist.h>
 #include <qvaluelist.h>
+#include <qlayout.h>
 
-#include "../image.h"
-#include "../mixednumber.h"
+#include "image.h"
+#include "datablocks/mixednumber.h"
 #include "dialogs/borderdialog.h"
+//#include "dialogs/krepagelayoutdia.h"
+//#include "widgets/kreruler.h"
 
 #include <cmath>
 
-SetupDisplay::SetupDisplay( const Recipe &sample, QWidget *parent ) : DragArea( parent, "background" ),
+SetupDisplay::SetupDisplay( const Recipe &sample, bool show_ruler, QWidget *parent ) : QWidget(parent),
+		hruler(0), vruler(0),
 		box_properties( new PropertiesMap ),
 		widget_item_map( new QMap<QWidget*, KreDisplayItem*> ),
 		has_changes( false )
 {
-	setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
+	QGridLayout *layout = new QGridLayout(this);
 
-	createItem( this, BackgroundColor );
+	drag_area = new DragArea( this, "background" );
+	drag_area->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
 
-	connect( this, SIGNAL( widgetClicked( QMouseEvent*, QWidget* ) ), SLOT( widgetClicked( QMouseEvent*, QWidget* ) ) );
-	connect( this, SIGNAL( widgetGeometryChanged() ), SLOT( changeMade() ) );
+	#if 0
+	if ( show_ruler ) {
+		page_layout = KoPageLayout::standardLayout();
+	
+		hruler = new KoRuler(this,drag_area,Qt::Horizontal,page_layout,0,KoUnit::U_INCH);
+		vruler = new KoRuler(this,drag_area,Qt::Vertical,page_layout,0,KoUnit::U_INCH);
+	
+		layout->addWidget(hruler,0,1);
+		layout->addWidget(vruler,1,0);
 
-	createWidgets( sample );
+		QWidget *behind_paper = new QWidget(this);
+		behind_paper->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
 
-	KConfig *config = kapp->config();
-	config->setGroup( "Page Setup" );
-	QSize default_size( 300, 400 );
-	m_size = config->readSizeEntry( "WindowSize", &default_size );
-	resize( m_size );
-	parentWidget() ->resize( m_size );
+		drag_area->reparent(behind_paper,QPoint(0,0),true);
+		layout->addWidget(behind_paper,1,1);
+
+		connect( hruler, SIGNAL(doubleClicked()), SLOT(openPageLayoutDia()) );
+		connect( vruler, SIGNAL(doubleClicked()), SLOT(openPageLayoutDia()) );
+
+		connect( hruler, SIGNAL(newPageLayout(const KoPageLayout&)), SLOT(updatePageLayout(const KoPageLayout&)) );
+		connect( vruler, SIGNAL(newPageLayout(const KoPageLayout&)), SLOT(updatePageLayout(const KoPageLayout&)) );
+	}
+	else
+	#endif
+		layout->addWidget(drag_area,0,0);
+
+	drag_area->resize(parentWidget()->size());
+
+	//FIXME: Where is this discrepancy of 4 coming from??
+	drag_area->resize( drag_area->width()-4, drag_area->height() );
+
+	createItem( drag_area, BackgroundColor );
+
+	connect( drag_area, SIGNAL( widgetClicked( QMouseEvent*, QWidget* ) ), SLOT( widgetClicked( QMouseEvent*, QWidget* ) ) );
+	connect( drag_area, SIGNAL( widgetGeometryChanged() ), SLOT( changeMade() ) );
+
+	createWidgets( sample, drag_area );
 }
 
 SetupDisplay::~SetupDisplay()
@@ -99,37 +132,44 @@ void SetupDisplay::loadLayout( const QString &filename )
 		for ( unsigned i = 0 ; i < l.count(); i++ ) {
 			QDomElement el = l.item( i ).toElement();
 			QString tagName = el.tagName();
-
-			QMap<QString, KreDisplayItem*>::iterator map_it = widget_map.find( tagName );
-			if ( map_it != widget_map.end() ) {
-				QDomNodeList l = el.childNodes();
-				for ( unsigned i = 0 ; i < l.count(); i++ ) {
-					QDomElement el = l.item( i ).toElement();
-					QString subTagName = el.tagName();
-
-					if ( subTagName == "background-color" )
-						loadBackgroundColor( *map_it, el );
-					else if ( subTagName == "geometry" )
-						loadGeometry( *map_it, el );
-					else if ( subTagName == "font" )
-						loadFont( *map_it, el );
-					else if ( subTagName == "text-color" )
-						loadTextColor( *map_it, el );
-					else if ( subTagName == "visible" )
-						loadVisibility( *map_it, el );
-					else if ( subTagName == "alignment" )
-						loadAlignment( *map_it, el );
-					else if ( subTagName == "border" )
-						loadBorder( *map_it, el );
-					else
-						kdDebug() << "Warning: Unknown tag within <" << tagName << ">: " << subTagName << endl;
+			if ( tagName == "page-layout-properties" ) {
+				//page_layout.loadKreFormat( el );
+				//updatePageLayout(page_layout,false);
+			}
+			else {
+				QMap<QString, KreDisplayItem*>::iterator map_it = widget_map.find( tagName );
+				if ( map_it != widget_map.end() ) {
+					QDomNodeList l = el.childNodes();
+					for ( unsigned i = 0 ; i < l.count(); i++ ) {
+						QDomElement el = l.item( i ).toElement();
+						QString subTagName = el.tagName();
+	
+						if ( subTagName == "background-color" )
+							loadBackgroundColor( *map_it, el );
+						else if ( subTagName == "geometry" )
+							loadGeometry( *map_it, el );
+						else if ( subTagName == "font" )
+							loadFont( *map_it, el );
+						else if ( subTagName == "overflow" )
+							loadOverflow( *map_it, el );
+						else if ( subTagName == "text-color" )
+							loadTextColor( *map_it, el );
+						else if ( subTagName == "visible" )
+							loadVisibility( *map_it, el );
+						else if ( subTagName == "alignment" )
+							loadAlignment( *map_it, el );
+						else if ( subTagName == "border" )
+							loadBorder( *map_it, el );
+						else
+							kdDebug() << "Warning: Unknown tag within <" << tagName << ">: " << subTagName << endl;
+					}
 				}
+				else
+					kdDebug() << "Warning: Unknown tag within <krecipes-layout>: " << tagName << endl;
 			}
-			else
-				kdDebug() << "Warning: Unknown tag within <krecipes-layout>: " << tagName << endl;
 		}
 
-		setWidget( 0 );
+		drag_area->setWidget( 0 );
 	}
 	else
 		kdDebug() << "Unable to open file: " << filename << endl;
@@ -149,6 +189,11 @@ void SetupDisplay::loadFont( KreDisplayItem *item, const QDomElement &tag )
 		item->widget->setFont( f );
 }
 
+void SetupDisplay::loadOverflow( KreDisplayItem *item, const QDomElement &tag )
+{
+	item->overflow = KreDisplayItem::OverflowType(tag.text().toInt());
+}
+
 void SetupDisplay::loadGeometry( KreDisplayItem *item, const QDomElement &tag )
 {
 	PreciseRect r( tag.attribute( "left" ).toDouble(), tag.attribute( "top" ).toDouble(), tag.attribute( "width" ).toDouble(), tag.attribute( "height" ).toDouble() );
@@ -196,6 +241,9 @@ void SetupDisplay::saveLayout( const QString &filename )
 	//layout_tag.setAttribute( "generator", QString("Krecipes v%1").arg(krecipes_version()) );
 	doc.appendChild( layout_tag );
 
+	//if ( hruler ) //a ruler was created if we are using the page layout
+	//	layout_tag.appendChild( page_layout.saveKreFormat(doc) );
+
 	for ( PropertiesMap::const_iterator it = box_properties->begin(); it != box_properties->end(); ++it ) {
 		QDomElement base_tag = doc.createElement( it.key() ->widget->name() );
 		layout_tag.appendChild( base_tag );
@@ -218,6 +266,12 @@ void SetupDisplay::saveLayout( const QString &filename )
 			base_tag.appendChild( font_tag );
 		}
 
+		if ( it.data() & Overflow ) {
+			QDomElement font_tag = doc.createElement( "overflow" );
+			font_tag.appendChild( doc.createTextNode( QString::number(it.key()->overflow) ) );
+			base_tag.appendChild( font_tag );
+		}
+
 		if ( it.data() & Visibility ) {
 			QDomElement visibility_tag = doc.createElement( "visible" );
 			visibility_tag.appendChild( doc.createTextNode( ( it.key() ->widget->isShown() ) ? "true" : "false" ) );
@@ -251,11 +305,6 @@ void SetupDisplay::saveLayout( const QString &filename )
 		}
 	}
 
-	KConfig *config = kapp->config();
-	config->setGroup( "Page Setup" );
-	config->writeEntry( "Aspect", static_cast<double>( width() ) / static_cast<double>( height() ) );
-	config->writeEntry( "WindowSize", parentWidget() ->size() );
-
 	QFile out_file( filename );
 	if ( out_file.open( IO_WriteOnly ) ) {
 		has_changes = false;
@@ -267,7 +316,7 @@ void SetupDisplay::saveLayout( const QString &filename )
 		kdDebug() << "Error: Unable to write to file " << filename << endl;
 }
 
-void SetupDisplay::createWidgets( const Recipe &sample )
+void SetupDisplay::createWidgets( const Recipe &sample, DragArea *canvas )
 {
 	//=========================TITLE=======================//
 	QString title;
@@ -276,7 +325,7 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 	else
 		title = sample.title;
 
-	title_box = new QLabel( title, this, "title" );
+	title_box = new QLabel( title, canvas, "title" );
 	title_box->setFrameShape( QFrame::Box );
 	title_box->setMinimumSize( 5, 5 );
 	title_box->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
@@ -293,7 +342,7 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 		instr = sample.instructions;
 
 	instr.replace( "\n", "<BR>" );
-	instr_box = new QLabel( instr, this, "instructions" );
+	instr_box = new QLabel( instr, canvas, "instructions" );
 	instr_box->setMinimumSize( 5, 5 );
 	instr_box->setFrameShape( QFrame::Box );
 	instr_box->setAlignment( Qt::AlignTop );
@@ -301,18 +350,18 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 	QToolTip::add
 		( instr_box, i18n( "Instructions" ) );
 
-	createItem( instr_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
+	createItem( instr_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
 
 	//=======================SERVINGS======================//
-	servings_box = new QLabel( QString( "<b>%1:</b> %2" ).arg( i18n( "Servings" ) ).arg( sample.persons ), this, "servings" );
+	servings_box = new QLabel( QString( "<b>%1:</b> %2" ).arg( i18n( "Yield" ) ).arg( sample.yield.amount ), canvas, "servings" );
 	servings_box->setFrameShape( QFrame::Box );
 	QToolTip::add
-		( servings_box, i18n( "Servings" ) );
+		( servings_box, i18n( "Yield" ) );
 
 	createItem( servings_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
 
 	//=======================PREP TIME======================//
-	preptime_box = new QLabel( QString( "<b>%1:</b> %2" ).arg( i18n( "Preparation Time" ) ).arg( sample.prepTime.toString( "h:mm" ) ), this, "prep_time" );
+	preptime_box = new QLabel( QString( "<b>%1:</b> %2" ).arg( i18n( "Preparation Time" ) ).arg( sample.prepTime.toString( "h:mm" ) ), canvas, "prep_time" );
 	preptime_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( preptime_box, i18n( "Preparation Time" ) );
@@ -320,7 +369,7 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 	createItem( preptime_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
 
 	//========================PHOTO========================//
-	photo_box = new QLabel( this, "photo" );
+	photo_box = new QLabel( canvas, "photo" );
 	photo_box->setFrameShape( QFrame::Box );
 	photo_box->setMinimumSize( 5, 5 );
 	photo_box->setScaledContents( true );
@@ -345,12 +394,12 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 		authors = i18n( "Author 1, Author 2, ..." );
 	authors.prepend( QString( "<b>%1: </b>" ).arg( i18n( "Authors" ) ) );
 
-	authors_box = new QLabel( authors, this, "authors" );
+	authors_box = new QLabel( authors, canvas, "authors" );
 	authors_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( authors_box, i18n( "Authors" ) );
 
-	createItem( authors_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
+	createItem( authors_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
 
 	//========================CATEGORIES========================//
 	QString categories;
@@ -364,15 +413,15 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 		categories = i18n( "Category 1, Category 2, ..." );
 	categories.prepend( QString( "<b>%1: </b>" ).arg( i18n( "Categories" ) ) );
 
-	categories_box = new QLabel( categories, this, "categories" );
+	categories_box = new QLabel( categories, canvas, "categories" );
 	categories_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( categories_box, i18n( "Categories" ) );
 
-	createItem( categories_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
+	createItem( categories_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
 
 	//=======================ID======================//
-	id_box = new QLabel( QString( i18n( "Recipe: #%1" ) ).arg( sample.recipeID ), this, "header" );
+	id_box = new QLabel( QString( i18n( "Recipe: #%1" ) ).arg( sample.recipeID ), canvas, "header" );
 	id_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( id_box, i18n( "Header" ) );
@@ -400,8 +449,8 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 		tmp_format.replace( QRegExp( QString::fromLatin1( "%n" ) ), ( *ing_it ).name );
 		tmp_format.replace( QRegExp( QString::fromLatin1( "%a" ) ), amount_str );
 		tmp_format.replace( QRegExp( QString::fromLatin1( "%u" ) ), ( *ing_it ).amount > 1 ? ( *ing_it ).units.plural : ( *ing_it ).units.name );
-		tmp_format.replace( QRegExp( QString::fromLatin1( "%p" ) ), ( ( *ing_it ).prepMethod.isEmpty() ) ?
-		                    QString::fromLatin1( "" ) : QString::fromLatin1( "; " ) + ( *ing_it ).prepMethod );
+		tmp_format.replace( QRegExp( QString::fromLatin1( "%p" ) ), ( ( *ing_it ).prepMethodList.count() == 0 ) ?
+		                    QString::fromLatin1( "" ) : QString::fromLatin1( "; " ) + ( *ing_it ).prepMethodList.join(",") );
 
 		ingredients += QString( "<li>%1</li>" ).arg( tmp_format );
 	}
@@ -411,27 +460,35 @@ void SetupDisplay::createWidgets( const Recipe &sample )
 	else
 		ingredients = i18n( "<ul><li>Ingredient 1</li><li>Ingredient 2</li><li>...</li></ul>" );
 
-	ingredients_box = new QLabel( ingredients, this, "ingredients" );
+	ingredients_box = new QLabel( ingredients, canvas, "ingredients" );
 	ingredients_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( ingredients_box, i18n( "Ingredients" ) );
 
-	createItem( ingredients_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
+	createItem( ingredients_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
 
 	//========================PROPERTIES========================//
-	properties_box = new QLabel( i18n( "<ul><li>Property 1</li><li>Property 2</li><li>...</li></ul>" ), this, "properties" );
+	properties_box = new QLabel( i18n( "<ul><li>Property 1</li><li>Property 2</li><li>...</li></ul>" ), canvas, "properties" );
 	properties_box->setFrameShape( QFrame::Box );
 	QToolTip::add
 		( properties_box, i18n( "Properties" ) );
 
-	createItem( properties_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border );
+	createItem( properties_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
+
+	//========================RATINGS========================//
+	QLabel *ratings_box = new QLabel( QString("<b>%1:</b><br /><hr /><b>%2</b><br /><ul><li>%4</li><li>...</li></ul><i>%3</i>").arg(i18n("Ratings")).arg(i18n("Rater")).arg(i18n("Comments")).arg(i18n("Criteria: X stars")), canvas, "ratings" );
+	ratings_box->setFrameShape( QFrame::Box );
+	QToolTip::add
+		( ratings_box, i18n( "Ratings" ) );
+
+	createItem( ratings_box, Font | BackgroundColor | TextColor | Visibility | Geometry | Alignment | Border | Overflow );
 }
 
 void SetupDisplay::widgetClicked( QMouseEvent *e, QWidget *w )
 {
 	if ( e->button() == QMouseEvent::RightButton ) {
 		popup = new KPopupMenu( w ); //parent _must_ be widget acting on
-		popup->insertTitle( ( w == this ) ? i18n( "Background" ) : QToolTip::textFor( w ) );
+		popup->insertTitle( ( w == drag_area ) ? i18n( "Background" ) : QToolTip::textFor( w ) );
 
 		unsigned int properties = 0;
 		for ( PropertiesMap::const_iterator it = box_properties->begin(); it != box_properties->end(); ++it ) {
@@ -441,6 +498,8 @@ void SetupDisplay::widgetClicked( QMouseEvent *e, QWidget *w )
 			}
 		}
 
+		KIconLoader il;
+
 		if ( properties & BackgroundColor )
 			popup->insertItem( i18n( "Background Color..." ), this, SLOT( setBackgroundColor() ) );
 
@@ -448,7 +507,27 @@ void SetupDisplay::widgetClicked( QMouseEvent *e, QWidget *w )
 			popup->insertItem( i18n( "Text Color..." ), this, SLOT( setTextColor() ) );
 
 		if ( properties & Font )
-			popup->insertItem( i18n( "Font..." ), this, SLOT( setFont() ) );
+			popup->insertItem( il.loadIconSet( "text", KIcon::Small, 16 ), i18n( "Font..." ), this, SLOT( setFont() ) );
+
+		if ( properties & Overflow ) {
+			QPopupMenu * sub_popup = new QPopupMenu( popup );
+
+			QActionGroup *alignment_actions = new QActionGroup( this );
+			alignment_actions->setExclusive( true );
+
+			QAction *shrink_action = new QAction( i18n( "Shrink to Fit" ), i18n( "Shrink to Fit" ), 0, alignment_actions, 0, true );
+			QAction *grow_action = new QAction( i18n( "Grow" ), i18n( "Grow" ), 0, alignment_actions, 0, true );
+
+			bool shrink = (*widget_item_map->find( w ))->overflow == KreDisplayItem::ShrinkToFit;
+			shrink_action->setOn(shrink);
+			grow_action->setOn(!shrink);
+
+			connect( alignment_actions, SIGNAL( selected( QAction* ) ), SLOT( setOverflow( QAction* ) ) );
+
+			popup->insertItem( i18n( "Overflow" ), sub_popup );
+
+			alignment_actions->addTo( sub_popup );
+		}
 
 		if ( properties & Visibility ) {
 			int id = popup->insertItem( i18n( "Show" ), this, SLOT( setShown( int ) ) );
@@ -461,9 +540,17 @@ void SetupDisplay::widgetClicked( QMouseEvent *e, QWidget *w )
 			QActionGroup *alignment_actions = new QActionGroup( this );
 			alignment_actions->setExclusive( true );
 
-			new QAction( i18n( "Center" ), i18n( "Center" ), 0, alignment_actions, 0, true );
-			new QAction( i18n( "Left" ), i18n( "Left" ), 0, alignment_actions, 0, true );
-			new QAction( i18n( "Right" ), i18n( "Right" ), 0, alignment_actions, 0, true );
+			QAction *c_action = new QAction( i18n( "Center" ), i18n( "Center" ), 0, alignment_actions, 0, true );
+			QAction *l_action = new QAction( i18n( "Left" ), i18n( "Left" ), 0, alignment_actions, 0, true );
+			QAction *r_action = new QAction( i18n( "Right" ), i18n( "Right" ), 0, alignment_actions, 0, true );
+
+			int align = ((QLabel*)w)->alignment();
+			if ( align & Qt::AlignHCenter )
+				c_action->setOn(true);
+			if ( align & Qt::AlignLeft )
+				l_action->setOn(true);
+			if ( align & Qt::AlignRight )
+				r_action->setOn(true);
 
 			connect( alignment_actions, SIGNAL( selected( QAction* ) ), SLOT( setAlignment( QAction* ) ) );
 
@@ -549,7 +636,7 @@ void SetupDisplay::setShown( int id )
 		box->setShown( !popup->isItemChecked( id ) );
 		emit itemVisibilityChanged( box, !popup->isItemChecked( id ) );
 
-		setWidget( 0 );
+		drag_area->setWidget( 0 );
 	}
 }
 
@@ -583,12 +670,25 @@ void SetupDisplay::setAlignment( QAction *action )
 	}
 }
 
+void SetupDisplay::setOverflow( QAction *action )
+{
+	QObject * obj = popup->parent();
+	if ( obj->inherits( "QLabel" ) ) {
+		has_changes = true;
+
+		QLabel *box = static_cast<QLabel*>( obj );
+		KreDisplayItem *item = *widget_item_map->find( box );
+
+		item->overflow = (action->text() == i18n("Grow"))?KreDisplayItem::Grow : KreDisplayItem::ShrinkToFit;
+	}
+}
+
 void SetupDisplay::setItemShown( QWidget *item, bool visible )
 {
 	item->raise(); //raise it to make sure the user can see it (and doesn't think it didn't work)
 	item->setShown( visible );
 
-	setWidget( 0 );
+	drag_area->setWidget( 0 );
 
 	has_changes = true;
 }
@@ -599,11 +699,11 @@ void SetupDisplay::setItemShown( QWidget *item, bool visible )
 
 void SetupDisplay::toAbsolute( PreciseRect *r )
 {
-	r->setWidth( r->width() / 100.0 * width() );
-	r->setHeight( r->height() / 100.0 * width() );
+	r->setWidth( r->width() / 100.0 * drag_area->width() );
+	r->setHeight( r->height() / 100.0 * drag_area->width() );
 
-	r->setLeft( r->left() / 100.0 * width() );
-	r->setTop( r->top() / 100.0 * width() );
+	r->setLeft( r->left() / 100.0 * drag_area->width() );
+	r->setTop( r->top() / 100.0 * drag_area->width() );
 }
 
 /*
@@ -612,11 +712,11 @@ void SetupDisplay::toAbsolute( PreciseRect *r )
 
 void SetupDisplay::toPercentage( PreciseRect *r )
 {
-	r->setWidth( r->width() * 100.0 / width() );
-	r->setHeight( r->height() * 100.0 / width() );
+	r->setWidth( r->width() * 100.0 / drag_area->width() );
+	r->setHeight( r->height() * 100.0 / drag_area->width() );
 
-	r->setLeft( r->left() * 100.0 / width() );
-	r->setTop( r->top() * 100.0 / width() );
+	r->setLeft( r->left() * 100.0 / drag_area->width() );
+	r->setTop( r->top() * 100.0 / drag_area->width() );
 }
 
 QSize SetupDisplay::minimumSize() const
@@ -634,4 +734,34 @@ void SetupDisplay::changeMade( void )
 	has_changes = true;
 }
 
+void SetupDisplay::openPageLayoutDia()
+{
+	#if 0
+	KoHeadFoot hf;
+	KoPageLayoutDia::pageLayout( page_layout, hf, (int)FORMAT_AND_BORDERS, unit, this );
+	updatePageLayout( page_layout );
+	#endif
+}
+
+#if 0
+void SetupDisplay::updatePageLayout( const KoPageLayout &new_page_layout, bool set_change )
+{
+	if ( hruler && vruler ) {
+		page_layout = new_page_layout;
+
+		hruler->setPageLayout( page_layout );
+		vruler->setPageLayout( page_layout );
+
+		//hruler->setFrameStartEnd( 0, page_layout.ptWidth );
+		//vruler->setFrameStartEnd( 0, page_layout.ptHeight );
+	
+		drag_area->setGeometry( QRect( QPoint(page_layout.ptLeft,page_layout.ptTop),
+			QSize(page_layout.ptWidth-page_layout.ptRight,page_layout.ptHeight-page_layout.ptBottom) ) );
+
+		if ( set_change )
+			changeMade();
+	}
+}
+#endif
+
 #include "setupdisplay.moc"
