@@ -18,6 +18,7 @@
 #include <qdir.h>
 #include <qstylesheet.h> //for QStyleSheet::escape() to escape for HTML
 #include <dom/dom_element.h>
+#include <qpainter.h>
 
 #include <kconfig.h>
 #include <kdebug.h>
@@ -28,11 +29,13 @@
 #include <kprogress.h>
 #include <kstandarddirs.h>
 #include <kurl.h>
+#include <kiconloader.h>
 
 #include "propertycalculator.h"
-#include "mixednumber.h"
+#include "datablocks/mixednumber.h"
 #include "dialogs/setupdisplay.h"
 #include "image.h"
+#include "krepagelayout.h"
 
 #include <cmath> //for ceil()
 
@@ -44,14 +47,48 @@ HTMLExporter::HTMLExporter( RecipeDB *db, const QString& filename, const QString
 	properties = new IngredientPropertyList;
 
 	offset = 0;
+
+	KConfig *config = KGlobal::config();
+	config->setGroup( "Page Setup" );
+
+	//let's do everything we can to be sure at least some layout is loaded
+	layout_filename = config->readEntry( "Layout", locate( "appdata", "layouts/default.klo" ) );
+	if ( layout_filename.isEmpty() || !QFile::exists( layout_filename ) )
+		layout_filename = locate( "appdata", "layouts/default.klo" );
+	kdDebug() << "Using layout file: " << layout_filename << endl;
 }
 
+#if 0
+HTMLExporter::HTMLExporter( RecipeDB *db, const QString& filename, const QString &format ) :
+		BaseExporter( filename, format ), database( db )
+{
+	div_elements.setAutoDelete( true );
+	properties = new IngredientPropertyList;
+
+	offset = 0;
+
+	KConfig *config = KGlobal::config();
+	config->setGroup( "Page Setup" );
+
+	//let's do everything we can to be sure at least some layout is loaded
+	layout_filename = config->readEntry( "PrintLayout", locate( "appdata", "layouts/default_print.klo" ) );
+	if ( layout_filename.isEmpty() || !QFile::exists( layout_filename ) )
+		layout_filename = locate( "appdata", "layouts/default_print.klo" );
+	kdDebug() << "Using layout file: " << layout_filename << endl;
+}
+#endif
+
 
 HTMLExporter::~HTMLExporter()
 {
 	delete properties;
 }
 
+int HTMLExporter::supportedItems() const
+{
+	return RecipeDB::All;
+}
+
 QString HTMLExporter::createContent( const RecipeList& recipes )
 {
 	RecipeList::const_iterator recipe_it;
@@ -77,14 +114,6 @@ QString HTMLExporter::createHeader( const RecipeList & )
 {
 	m_error = false;
 
-	KConfig *config = KGlobal::config();
-	config->setGroup( "Page Setup" );
-
-	//let's do everything we can to be sure at least some layout is loaded
-	QString layout_filename = config->readEntry( "Layout", locate( "appdata", "layouts/default.klo" ) );
-	if ( layout_filename.isEmpty() || !QFile::exists( layout_filename ) )
-		layout_filename = locate( "appdata", "layouts/default.klo" );
-	kdDebug() << "Using layout file: " << layout_filename << endl;
 	QFile input( layout_filename );
 
 	if ( !input.open( IO_ReadOnly ) ) {
@@ -108,6 +137,16 @@ QString HTMLExporter::createHeader( const RecipeList & )
 		return "<html>"+error_str+"</html>";
 	}
 
+	#if 0
+	QDomNodeList node_list = doc.elementsByTagName( "page-layout-properties" );
+	if ( node_list.count() > 0 ) {
+		QDomElement layout_el = node_list.item( 0 ).toElement();
+		KoPageLayout page_layout; page_layout.loadKreFormat(layout_el);
+		m_width = page_layout.ptWidth;
+		kdDebug()<<"Setting width to "<<m_width<<endl;
+	}
+	#endif
+
 	//put all the recipe photos into this directory
 	QDir dir;
 	QFileInfo fi(fileName());
@@ -144,7 +183,7 @@ QString HTMLExporter::createFooter()
 	//and now piece it all together
 	QString recipeHTML = "<html>\n<head>\n";
 	recipeHTML += "<meta name=\"lang\" content=\"" + loc->language() + "\">\n";
-	recipeHTML += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=" + encoding.replace( " ", "-" ) + "\" />\n";
+	recipeHTML += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
 	recipeHTML += QString( "<title>%1</title>" ).arg( i18n( "Krecipes Recipes" ) );
 	recipeHTML += recipeStyleHTML;
 	recipeHTML += "</head>\n";
@@ -229,48 +268,45 @@ int HTMLExporter::createBlocks( const Recipe &recipe, const QDomDocument &doc, i
 	for ( QRect * rect = geometries.first(); rect; rect = geometries.next() ) {
 		DivElement * element = geom_contents.find( rect );
 
-		// For those elements that have no fixed height (lists), calculate the height
+		// For those elements that have no fixed height (lists), calculate the height or shrink it to fit
 		int elementHeight = ( int ) ( rect->height() / 100.0 * m_width ); //Initialize with the current user settings
+		int font_size = -1;
 		if ( !element->fixedHeight() ) {
-			int elementWidth = ( int ) ( rect->width() / 100.0 * m_width );
+			QDomElement el = getLayoutAttribute( doc, element->className(), "overflow" );
+			if ( !el.isNull() && el.text().toInt() == 0 ) {
+				font_size = 12; //default if none is set by the user
+				QDomElement el = getLayoutAttribute( doc, element->className(), "font" );
+				if ( !el.isNull() ) {
+					QFont font;
+					font.fromString( el.text() );
+					font_size = font.pointSize();
+				}
 
-			// Generate a test page to calculate the size in khtml
-			QString tempHTML = "<html><head><style type=\"text/css\">";
-			tempHTML += classesCSS;
-			tempHTML += QString( "#%1 {" ).arg( element->id() );
-			tempHTML += QString( "width: %1px;" ).arg( elementWidth );
-			tempHTML += "} ";
-			tempHTML += "body { margin: 0px; }"; //very important subtlety in determining the exact width
-			tempHTML += "</style></head>";
-			tempHTML += "<body>";
-			tempHTML += element->generateHTML();
-			tempHTML += "</body></html>";
+				int goal_height = elementHeight;
+				do {
+					font_size--;
+					elementHeight = getHeight( ( int ) ( rect->width() / 100.0 * m_width ), element, font_size );
+				}
+				while ( elementHeight > goal_height && font_size > 0 );
 
-			KHTMLPart *sizeCalculator = new KHTMLPart( ( QWidget* ) 0 );
-			sizeCalculator->view()->setVScrollBarMode ( QScrollView::AlwaysOff );
-			sizeCalculator->view()->setMinimumSize( QSize( elementWidth, 0 ) );
-			sizeCalculator->view()->resize( QSize( elementWidth, 0 ) );
-			sizeCalculator->begin( KURL( locateLocal( "tmp", "/" ) ) );
-			sizeCalculator->write( tempHTML );
-			sizeCalculator->end();
-
-			sizeCalculator->view()->layout(); //force a layout
-
-			// Set the size of the element
-			DOM::Document doc = sizeCalculator->document();
-			elementHeight = doc.getElementById( element->id() ).getRect().height();
-
-			delete sizeCalculator;
+				if ( font_size == 0 )
+					kdDebug()<<"Warning: Unable to 'Shrink to fit' - Not enough spacing available, so defaulting to the original font size."<<endl;
+			}
+			else
+				elementHeight = getHeight( ( int ) ( rect->width() / 100.0 * m_width ), element );
 		}
 		rect->setHeight( ( int ) ( ceil( elementHeight * 100.0 / m_width ) ) );
 
 		// Move elements around if there's any overlapping
 		pushItemsDownIfNecessary( geometries, rect );
+		//geometries.sort(); //moving around items might have changed the order
 
 		element->addProperty( QString( "top: %1px;" ).arg( static_cast<int>( rect->top() / 100.0 * m_width ) ) );
 		element->addProperty( QString( "left: %1px;" ).arg( static_cast<int>( rect->left() / 100.0 * m_width ) ) );
 		element->addProperty( QString( "width: %1px;" ).arg( static_cast<int>( rect->width() / 100.0 * m_width ) ) );
 		element->addProperty( QString( "height: %1px;" ).arg( elementHeight ) );
+		if ( font_size > 0 )
+			element->addProperty( QString( "font-size: %1pt;" ).arg( font_size ) );
 
 		height_taken = QMAX( rect->top() + int( elementHeight * 100.0 / m_width ), height_taken );
 	}
@@ -278,28 +314,35 @@ int HTMLExporter::createBlocks( const Recipe &recipe, const QDomDocument &doc, i
 	return height_taken;
 }
 
+void HTMLExporter::insertHTMLIfVisible( QMap<QString, QString> &html_map, const QString &name, const QString &html )
+{
+	QDomElement el = getLayoutAttribute( doc, name, "visible" );
+	if ( el.isNull() || el.text() != "false" )
+		html_map.insert(name,html);
+}
+
 QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 {
 	KConfig * config = KGlobal::config();
 	QMap<QString, QString> html_map;
 
 	//=======================TITLE======================//
-	html_map.insert( "title", recipe.title );
+	insertHTMLIfVisible( html_map, "title", recipe.title );
 
 	//=======================INSTRUCTIONS======================//
 	QString instr_html = QStyleSheet::escape( recipe.instructions );
 	instr_html.replace( "\n", "<BR>" );
-	html_map.insert( "instructions", instr_html );
+	insertHTMLIfVisible( html_map, "instructions", instr_html );
 
 	//=======================SERVINGS======================//
-	QString servings_html = QString( "<b>%1: </b>%2" ).arg( i18n( "Servings" ) ).arg( recipe.persons );
-	html_map.insert( "servings", servings_html );
+	QString servings_html = QString( "<b>%1: </b>%2" ).arg( i18n( "Yield" ) ).arg( recipe.yield.toString() );
+	insertHTMLIfVisible( html_map, "servings", servings_html );
 
 	//=======================PREP TIME======================//
 	QString preptime_html;
 	if ( !recipe.prepTime.isNull() && recipe.prepTime.isValid() )
 		preptime_html = QString( "<b>%1: </b>%2" ).arg( i18n( "Preparation Time" ) ).arg( recipe.prepTime.toString( "h:mm" ) );
-	html_map.insert( "prep_time", preptime_html );
+	insertHTMLIfVisible( html_map, "prep_time", preptime_html );
 
 	//========================PHOTO========================//
 	QString photo_name;
@@ -312,7 +355,7 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 	QString image_url = fi.baseName() + "_photos/" + escape( photo_name ) + ".png";
 	image_url = KURL::encode_string( image_url );
 	QString photo_html = QString( "<img src=\"%1\">" ).arg( image_url );
-	html_map.insert( "photo", photo_html );
+	insertHTMLIfVisible( html_map, "photo", photo_html );
 
 	//=======================AUTHORS======================//
 	QString authors_html;
@@ -326,7 +369,7 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 	}
 	if ( !authors_html.isEmpty() )
 		authors_html.prepend( QString( "<b>%1: </b>" ).arg( i18n( "Authors" ) ) );
-	html_map.insert( "authors", authors_html );
+	insertHTMLIfVisible( html_map, "authors", authors_html );
 
 	//=======================CATEGORIES======================//
 	QString categories_html;
@@ -341,11 +384,11 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 	if ( !categories_html.isEmpty() )
 		categories_html.prepend( QString( "<b>%1: </b>" ).arg( i18n( "Categories" ) ) );
 
-	html_map.insert( "categories", categories_html );
+	insertHTMLIfVisible( html_map, "categories", categories_html );
 
 	//=======================HEADER======================//
 	QString header_html = QString( "<b>%1 #%2</b>" ).arg( i18n( "Recipe" ) ).arg( recipe.recipeID );
-	html_map.insert( "header", header_html );
+	insertHTMLIfVisible( html_map, "header", header_html );
 
 	//=======================INGREDIENTS======================//
 	QString ingredients_html;
@@ -364,15 +407,17 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 		for ( IngredientList::const_iterator ing_it = group_list.begin(); ing_it != group_list.end(); ++ing_it ) {
 			QString amount_str = MixedNumber( ( *ing_it ).amount ).toString( number_format );
 
-			if ( ( *ing_it ).amount <= 1e-10 )
+			if ( (*ing_it).amount_offset > 0 )
+				amount_str += "-"+MixedNumber( ( *ing_it ).amount + ( *ing_it ).amount_offset ).toString( number_format );
+			else if ( ( *ing_it ).amount <= 1e-10 )
 				amount_str = "";
 
 			QString tmp_format( ingredient_format );
 			tmp_format.replace( QRegExp( QString::fromLatin1( "%n" ) ), QStyleSheet::escape( ( *ing_it ).name ) );
 			tmp_format.replace( QRegExp( QString::fromLatin1( "%a" ) ), amount_str );
-			tmp_format.replace( QRegExp( QString::fromLatin1( "%u" ) ), QStyleSheet::escape( ( ( *ing_it ).amount > 1 ) ? ( *ing_it ).units.plural : ( *ing_it ).units.name ) );
-			tmp_format.replace( QRegExp( QString::fromLatin1( "%p" ) ), ( ( *ing_it ).prepMethod.isEmpty() ) ?
-			                    QString::fromLatin1( "" ) : QString::fromLatin1( "; " ) + QStyleSheet::escape( ( *ing_it ).prepMethod ) );
+			tmp_format.replace( QRegExp( QString::fromLatin1( "%u" ) ), QStyleSheet::escape( ( ( *ing_it ).amount + ( *ing_it ).amount_offset > 1 ) ? ( *ing_it ).units.plural : ( *ing_it ).units.name ) );
+			tmp_format.replace( QRegExp( QString::fromLatin1( "%p" ) ), ( ( *ing_it ).prepMethodList.count() == 0 ) ?
+			                    QString::fromLatin1( "" ) : QString::fromLatin1( "; " ) + QStyleSheet::escape( ( *ing_it ).prepMethodList.join(",") ) );
 
 			ingredients_html += QString( "<li>%1</li>" ).arg( tmp_format );
 		}
@@ -384,7 +429,7 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 		ingredients_html.prepend( "<ul>" );
 		ingredients_html.append( "</ul>" );
 	}
-	html_map.insert( "ingredients", ingredients_html );
+	insertHTMLIfVisible( html_map, "ingredients", ingredients_html );
 
 	//=======================PROPERTIES======================//
 	QString properties_html;
@@ -415,7 +460,38 @@ QMap<QString, QString> HTMLExporter::generateBlocksHTML( const Recipe &recipe )
 		properties_html.prepend( "<ul>" );
 		properties_html.append( "</ul>" );
 	}
-	html_map.insert( "properties", properties_html );
+	insertHTMLIfVisible( html_map, "properties", properties_html );
+
+	//=======================RATINGS======================//
+	QString ratings_html;
+	if ( recipe.ratingList.count() > 0 )
+		ratings_html += QString("<b>%1:</b>").arg(i18n("Ratings"));
+
+	for ( RatingList::const_iterator rating_it = recipe.ratingList.begin(); rating_it != recipe.ratingList.end(); ++rating_it ) {
+		ratings_html += "<hr />";
+
+		if ( !( *rating_it ).rater.isEmpty() )
+			ratings_html += "<p><b>"+( *rating_it ).rater+"</b></p>";
+
+		if ( (*rating_it).ratingCriteriaList.count() > 0 )
+			ratings_html += "<table>";
+		for ( RatingCriteriaList::const_iterator rc_it = (*rating_it).ratingCriteriaList.begin(); rc_it != (*rating_it).ratingCriteriaList.end(); ++rc_it ) {
+			QString image_url = fi.baseName() + "_photos/" + QString::number((*rc_it).stars) + "-stars.png";
+			image_url = KURL::encode_string( image_url );
+			ratings_html +=  "<tr><td>"+(*rc_it).name+":</td><td><img src=\""+image_url+"\" /></td></tr>";
+			if ( !QFile::exists( fi.dirPath(true) + "/" + image_url ) ) {
+				QPixmap starPixmap = Rating::starsPixmap((*rc_it).stars,true);
+				starPixmap.save( fi.dirPath(true) + "/" + image_url, "PNG" );
+			}
+			
+		}
+		if ( (*rating_it).ratingCriteriaList.count() > 0 )
+			ratings_html += "</table>";
+
+		if ( !( *rating_it ).comment.isEmpty() )
+			ratings_html += "<p><i>"+( *rating_it ).comment+"</i></p>";
+	}
+	insertHTMLIfVisible( html_map, "ratings", ratings_html );
 
 	///////////TODO?: Add an "end of recipe" element here (as a separator between this and the next recipes//////////////
 
@@ -518,8 +594,8 @@ QString HTMLExporter::readFontProperties( const QDomDocument &doc, const QString
 		font.fromString( el.text() );
 
 		text += QString( "font-family: %1;\n" ).arg( font.family() );
-		text += QString( "font-size: %1pt;\n" ).arg( font.pointSize() );
 		text += QString( "font-weight: %1;\n" ).arg( font.weight() );
+		text += QString( "font-size: %1pt;\n" ).arg( font.pointSize() );
 
 		return text;
 	}
@@ -562,6 +638,15 @@ void HTMLExporter::pushItemsDownIfNecessary( QPtrList<QRect> &geometries, QRect
 		if ( intersection.isValid() ) {
 			height_offset = QABS( intersection.top() - top_geom->bottom() );
 			item->moveBy( 0, height_offset + 5 );
+
+			#if 0
+			QRect *save_pos = item;
+			for ( item = geometries.next(); item; item = geometries.next() ) {
+				item->moveBy( 0, height_offset + 5 );
+			}
+			geometries.findRef( save_pos );
+			#endif
+			//geometries.prev();
 		}
 	}
 
@@ -596,6 +681,10 @@ void HTMLExporter::removeHTMLFiles( const QString &filename, const QStringList &
 	//remove photo directory
 	QDir photo_dir;
 	photo_dir.rmdir( filename + "_photos" );
+
+	for ( double d = 0.5; d < 5.5; d += 0.5 ) {
+		if ( QFile::exists(filename+"_photos/"+QString::number(d)+"-stars.png") ) photo.remove(filename+"_photos/"+QString::number(d)+"-stars.png");
+	}
 }
 
 QDomElement HTMLExporter::getLayoutAttribute( const QDomDocument &doc, const QString &object, const QString &attribute )
@@ -626,6 +715,44 @@ QString HTMLExporter::escape( const QString & str )
 	return tmp.replace( '/', "_" );
 }
 
+int HTMLExporter::getHeight( int constrained_width, DivElement *element, int font_size )
+{
+	// Generate a test page to calculate the size in khtml
+	QString tempHTML = "<html><head><style type=\"text/css\">";
+	tempHTML += classesCSS;
+	tempHTML += QString( "#%1 {" ).arg( element->id() );
+	tempHTML += QString( "width: %1px;" ).arg( constrained_width );
+	if ( font_size > 0 )
+		tempHTML += QString( "font-size: %1pt;" ).arg( font_size );
+	tempHTML += "} ";
+	tempHTML += "body { margin: 0px; }"; //very important subtlety in determining the exact width
+	tempHTML += "</style></head>";
+	tempHTML += "<body>";
+	tempHTML += element->generateHTML();
+	tempHTML += "</body></html>";
+
+	KHTMLPart *sizeCalculator = new KHTMLPart( ( QWidget* ) 0 );
+	sizeCalculator->view()->setVScrollBarMode ( QScrollView::AlwaysOff );
+	sizeCalculator->view()->setMinimumSize( QSize( constrained_width, 0 ) );
+	sizeCalculator->view()->resize( QSize( constrained_width, 0 ) );
+	sizeCalculator->begin( KURL( locateLocal( "tmp", "/" ) ) );
+	sizeCalculator->write( tempHTML );
+	sizeCalculator->end();
+
+	sizeCalculator->view()->layout(); //force a layout
+
+	// Set the size of the element
+	DOM::Document size_test_doc = sizeCalculator->document();
+	int return_height = size_test_doc.getElementById( element->id() ).getRect().height();
+
+	if ( size_test_doc.getElementById( element->id() ).getRect().width() != constrained_width )
+		kdDebug()<<"Warning: Element expanded past the constrained width"<<endl;
+
+	delete sizeCalculator;
+
+	return return_height;
+}
+
 
 /////////////////////////   CustomRectList   //////////////////////////////////
 
