@@ -1,5 +1,5 @@
 /***************************************************************************
-*   Copyright (C) 2003 by                                                 *
+*   Copyright (C) 2003-2005 by                                            *
 *   Jason Kivlighn (mizunoami44@users.sourceforge.net)                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
@@ -31,7 +31,7 @@ class CategoryTree;
 /**
 Import for Krecipes native file format (.kre, .kreml)
  
-@author Cyril Bosselut
+@author Cyril Bosselut, Jason Kivlighn
 */
 class KreImporter: public BaseImporter
 {
@@ -46,6 +46,9 @@ private:
 	void readCategoryStructure( const QDomNodeList& l, CategoryTree *tree );
 	void readDescription( const QDomNodeList& l, Recipe* );
 	void readIngredients( const QDomNodeList& l, Recipe*, const QString &header = QString::null );
+	void readAmount( const QDomElement& amount, double &amount, double &amount_offset );
+	void readRatings( const QDomNodeList&, Recipe * );
+	void readCriterion( const QDomNodeList&, RatingCriteriaList &r );
 };
 
 #endif
