@@ -1,3 +1,4 @@
+
 /***************************************************************************
 *   Copyright (C) 2003-2004 by                                            *
 *   Unai Garro (ugarro@users.sourceforge.net)                             *
@@ -52,6 +53,8 @@
 #include "widgets/kremenu.h"
 #include "widgets/paneldeco.h"
 
+#include "backends/progressinterface.h"
+
 #include "profiling.h"
 
 KrecipesView::KrecipesView( QWidget *parent )
@@ -295,6 +298,8 @@ KrecipesView::KrecipesView( QWidget *parent )
 
 	connect( rightPanel, SIGNAL( panelRaised( QWidget*, QWidget* ) ), SLOT( panelRaised( QWidget*, QWidget* ) ) );
 
+	connect( selectPanel, SIGNAL( recipeSelected(bool) ), SIGNAL( recipeSelected(bool) ) );
+
 	// Close Splash Screen
 	delete start_logo;
 
@@ -343,9 +348,6 @@ void KrecipesView::translate()
 
 void KrecipesView::print()
 {
-	// do the actual printing, here
-	// p->drawText(etc..)
-
 	viewPanel->print();
 }
 
@@ -424,32 +426,25 @@ bool KrecipesView::save( void )
     \fn KrecipesView::exportRecipe()
  */
 void KrecipesView::exportRecipe()
-{ /*
-		if ( !inputPanel->everythingSaved() )
-		{
-			Recipe *recipe;  selectPanel->getCurrentRecipe( recipe );
-			if ( recipe->recipeID == inputPanel->loadedRecipeID() )
-			{
-				switch( KMessageBox::questionYesNoCancel( this,
-				  i18n("This recipe has unsaved changes.\n"
-				  "In order to have these changes saved to a file, this recipe must first be saved to the database.\n"
-				  "Do you want to save this recipe to a file with or without these changes?"),
-				  i18n("Unsaved Changes") ) )
-				{
-				case KMessageBox::Yes: save();
-				case KMessageBox::No: selectPanel->slotExportRecipe();
-				case KMessageBox::Cancel: break;
-				default: break;
-				}
-			}
-		}
-		else*/
+{
 	QWidget * vis_panel = rightPanel->visiblePanel();
 	if ( vis_panel == viewPanel && viewPanel->recipesLoaded() > 0 ) {
 		exportRecipes( viewPanel->currentRecipes() );
 	}
 	else if ( vis_panel == selectPanel ) {
-		selectPanel->slotExportRecipe();
+		selectPanel->getActionsHandler()->recipeExport();
+	}
+}
+
+void KrecipesView::exportToClipboard()
+{
+	QWidget * vis_panel = rightPanel->visiblePanel();
+	if ( vis_panel == viewPanel && viewPanel->recipesLoaded() > 0 ) {
+		QValueList<int> ids = viewPanel->currentRecipes();
+		RecipeActionsHandler::recipesToClipboard( ids, database );
+	}
+	else if ( vis_panel == selectPanel ) {
+		selectPanel->getActionsHandler()->recipesToClipboard();
 	}
 }
 
@@ -494,7 +489,7 @@ void KrecipesView::actionRecipe( int recipeID, int action )
 	case 2:  //Remove
 		{
 			switch ( KMessageBox::questionYesNo( this,
-			                                     QString( i18n( "Are you sure you want to permanently remove the selected recipe?" ) ),
+			                                     QString( i18n( "Are you sure you want to permanently remove the recipe, %1?" ) ).arg(database->recipeTitle(recipeID)),
 			                                     i18n( "Confirm remove" ) ) )
 			{
 			case KMessageBox::Yes:
@@ -617,15 +612,16 @@ void KrecipesView::wizard( bool force )
 			if ( doUSDAImport ) {
 				// Open the DB first
 				setupWizard->getServerInfo( isRemote, host, client, dbName, user, pass ); //only used if needed by backend
-				RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName );
+				RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName, dbName );
 
 				// Import the data
 				if ( db ) {
 					db->connect();
 
 					if ( db->ok() ) {
-						KProgressDialog progress( this, "progress_dlg", i18n( "Nutrient Import" ), i18n( "Importing USDA nutrient data" ), true );
-						db->importUSDADatabase( &progress );
+						ProgressInterface pi(this);
+						pi.listenOn(db);
+						db->importUSDADatabase();
 					}
 
 					//close the database whether ok() or not
@@ -635,7 +631,7 @@ void KrecipesView::wizard( bool force )
 
 			//we can do a faster usda import if this is done after it
 			if ( initData ) {
-				RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName );
+				RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName, dbName );
 				if ( db ) {
 					db->connect();
 
@@ -671,9 +667,9 @@ void KrecipesView::setupUserPermissions( const QString &host, const QString &cli
 	else
 		kdDebug() << "Open db as:" << user << ",*** with password ****\n";
 
-	RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName );
+	RecipeDB *db = RecipeDB::createDatabase( dbType, host, user, pass, dbName, dbName );
 	if ( db ) {
-		db->connect();
+		db->connect(true,false);//create the database, but no tables (do that when connected as the user)
 		if ( db->ok() )
 			db->givePermissions( dbName, newUser, newPass, client ); // give permissions to the user
 		else
@@ -686,7 +682,7 @@ void KrecipesView::setupUserPermissions( const QString &host, const QString &cli
 
 void KrecipesView::initializeData( const QString &host, const QString &dbName, const QString &user, const QString &pass )
 {
-	RecipeDB * db = RecipeDB::createDatabase( dbType, host, user, pass, dbName );
+	RecipeDB * db = RecipeDB::createDatabase( dbType, host, user, pass, dbName, dbName );
 	if ( !db ) {
 		kdError() << i18n( "Code error. No DB support has been included. Exiting" ) << endl;
 		kapp->exit( 1 );
@@ -697,7 +693,6 @@ void KrecipesView::initializeData( const QString &host, const QString &dbName, c
 	if ( db->ok() ) {
 		db->emptyData();
 		db->initializeData();
-		db->importSamples();
 	}
 
 	delete db;
@@ -967,7 +962,7 @@ void KrecipesView::editRecipe()
 		actionRecipe( viewPanel->currentRecipes() [ 0 ], 1 );
 		break;
 	case SelectP:
-		selectPanel->getActionsHandler() ->edit();
+		selectPanel->getActionsHandler()->edit();
 		break;
 	default:
 		break;
