@@ -15,6 +15,10 @@
 
 #include "pref.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qhbox.h>
@@ -25,6 +29,7 @@
 #include <qtooltip.h>
 #include <qwhatsthis.h>
 #include <qframe.h>
+#include <qcombobox.h>
 
 #include <kapplication.h>
 #include <kconfig.h>
@@ -34,7 +39,7 @@
 #include <kfiledialog.h>
 #include <knuminput.h>
 #include <klineedit.h>
-
+#include <kurlrequester.h>
 
 KrecipesPreferences::KrecipesPreferences( QWidget *parent )
 		: KDialogBase( IconList, i18n( "Krecipes Preferences" ),
@@ -61,7 +66,7 @@ KrecipesPreferences::KrecipesPreferences( QWidget *parent )
 	m_pageNumbers = new NumbersPrefs( frame );
 	formatting_layout->addWidget( m_pageNumbers );
 
-	frame = addPage( i18n( "Import" ), i18n( "Recipe Import Options" ), il.loadIcon( "down", KIcon::NoGroup, 32 ) );
+	frame = addPage( i18n( "Import/Export" ), i18n( "Recipe Import and Export Options" ), il.loadIcon( "down", KIcon::NoGroup, 32 ) );
 	QHBoxLayout* import_layout = new QHBoxLayout( frame );
 	m_pageImport = new ImportPrefs( frame );
 	import_layout->addWidget( m_pageImport );
@@ -140,8 +145,28 @@ MySQLServerPrefs::MySQLServerPrefs( QWidget *parent ) : QWidget( parent )
 	dbNameEdit->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
 	layout->addWidget( dbNameEdit, 7, 2 );
 
+	QSpacerItem* spacerRow5 = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
+	layout->addItem( spacerRow5, 8, 1 );
+
+	// Backup options
+	QGroupBox *backupGBox = new QGroupBox( this, "backupGBox" );
+	backupGBox->setTitle( i18n( "Backup" ) );
+	backupGBox->setColumns( 2 );
+	layout->addMultiCellWidget( backupGBox, 9, 9, 1, 4 );
+
+	QLabel *dumpPathLabel = new QLabel( backupGBox );
+	dumpPathLabel->setText( QString(i18n( "Path to '%1':" )).arg("mysqldump") );
+	dumpPathRequester = new KURLRequester( backupGBox );
+	dumpPathRequester->setFilter( "mysqldump" );
+
+	QLabel *mysqlPathLabel = new QLabel( backupGBox );
+	mysqlPathLabel->setText( QString(i18n( "Path to '%1':" )).arg("mysql") );
+	mysqlPathRequester = new KURLRequester( backupGBox );
+	mysqlPathRequester->setFilter( "mysql" );
+
+
 	QSpacerItem* spacerRow4 = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
-	layout->addItem( spacerRow4, 8, 1 );
+	layout->addItem( spacerRow4, 10, 1 );
 	QSpacerItem* spacerRight = new QSpacerItem( 10, 10, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
 	layout->addItem( spacerRight, 1, 4 );
 
@@ -152,6 +177,8 @@ MySQLServerPrefs::MySQLServerPrefs( QWidget *parent ) : QWidget( parent )
 	usernameEdit->setText( config->readEntry( "Username", "" ) );
 	passwordEdit->setText( config->readEntry( "Password", "" ) );
 	dbNameEdit->setText( config->readEntry( "DBName", "Krecipes" ) );
+	dumpPathRequester->setURL( config->readEntry( "MySQLDumpPath", "mysqldump" ) );
+	mysqlPathRequester->setURL( config->readEntry( "MySQLPath", "mysql" ) );
 }
 
 void MySQLServerPrefs::saveOptions( void )
@@ -162,6 +189,8 @@ void MySQLServerPrefs::saveOptions( void )
 	config->writeEntry( "Username", usernameEdit->text() );
 	config->writeEntry( "Password", passwordEdit->text() );
 	config->writeEntry( "DBName", dbNameEdit->text() );
+	config->writeEntry( "MySQLDumpPath", dumpPathRequester->url() );
+	config->writeEntry( "MySQLPath", mysqlPathRequester->url() );
 }
 
 
@@ -226,8 +255,29 @@ PostgreSQLServerPrefs::PostgreSQLServerPrefs( QWidget *parent ) : QWidget( paren
 	dbNameEdit->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
 	layout->addWidget( dbNameEdit, 7, 2 );
 
+
+	QSpacerItem* spacerRow5 = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
+	layout->addItem( spacerRow5, 8, 1 );
+
+	// Backup options
+	QGroupBox *backupGBox = new QGroupBox( this, "backupGBox" );
+	backupGBox->setTitle( i18n( "Backup" ) );
+	backupGBox->setColumns( 2 );
+	layout->addMultiCellWidget( backupGBox, 9, 9, 1, 4 );
+
+	QLabel *dumpPathLabel = new QLabel( backupGBox );
+	dumpPathLabel->setText( QString(i18n( "Path to '%1':" )).arg("pg_dump") );
+	dumpPathRequester = new KURLRequester( backupGBox );
+	dumpPathRequester->setFilter( "pg_dump" );
+
+	QLabel *psqlPathLabel = new QLabel( backupGBox );
+	psqlPathLabel->setText( QString(i18n( "Path to '%1':" )).arg("psql") );
+	psqlPathRequester = new KURLRequester( backupGBox );
+	psqlPathRequester->setFilter( "psql" );
+
+
 	QSpacerItem* spacerRow4 = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
-	layout->addItem( spacerRow4, 8, 1 );
+	layout->addItem( spacerRow4, 10, 1 );
 	QSpacerItem* spacerRight = new QSpacerItem( 10, 10, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
 	layout->addItem( spacerRight, 1, 4 );
 
@@ -238,6 +288,8 @@ PostgreSQLServerPrefs::PostgreSQLServerPrefs( QWidget *parent ) : QWidget( paren
 	usernameEdit->setText( config->readEntry( "Username", "" ) );
 	passwordEdit->setText( config->readEntry( "Password", "" ) );
 	dbNameEdit->setText( config->readEntry( "DBName", "Krecipes" ) );
+	dumpPathRequester->setURL( config->readEntry( "PgDumpPath", "pg_dump" ) );
+	psqlPathRequester->setURL( config->readEntry( "PsqlPath", "psql" ) );
 }
 
 void PostgreSQLServerPrefs::saveOptions( void )
@@ -248,6 +300,8 @@ void PostgreSQLServerPrefs::saveOptions( void )
 	config->writeEntry( "Username", usernameEdit->text() );
 	config->writeEntry( "Password", passwordEdit->text() );
 	config->writeEntry( "DBName", dbNameEdit->text() );
+	config->writeEntry( "PgDumpPath", dumpPathRequester->url() );
+	config->writeEntry( "PsqlPath", psqlPathRequester->url() );
 }
 
 
@@ -259,38 +313,45 @@ SQLiteServerPrefs::SQLiteServerPrefs( QWidget *parent ) : QWidget( parent )
 	QHBox *hbox = new QHBox( this );
 	( void ) new QLabel( i18n( "Database file:" ), hbox );
 
-	fileEdit = new KLineEdit( hbox );
-	hbox->setStretchFactor( fileEdit, 2 );
-
-	KIconLoader il;
-	QPushButton *file_select = new QPushButton( il.loadIcon( "fileopen", KIcon::NoGroup, 16 ), QString::null, hbox );
-	QToolTip::add
-		( file_select, i18n( "Open file dialog" ) );
-	file_select->setFixedWidth( 25 );
+	fileRequester = new KURLRequester( hbox );
+	hbox->setStretchFactor( fileRequester, 2 );
 
 	Form1Layout->addWidget( hbox );
 
-	connect( file_select, SIGNAL( clicked() ), SLOT( selectFile() ) );
+	QSpacerItem* spacerRow5 = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
+	Form1Layout->addItem( spacerRow5 );
+
+	QString sqliteBinary;
+	#if HAVE_SQLITE3
+	sqliteBinary = "sqlite3";
+ 	#elif HAVE_SQLITE
+	sqliteBinary = "sqlite";
+	#endif
+
+	// Backup options
+	QGroupBox *backupGBox = new QGroupBox( this, "backupGBox" );
+	backupGBox->setTitle( i18n( "Backup" ) );
+	backupGBox->setColumns( 2 );
+	Form1Layout->addWidget( backupGBox );
+
+	QLabel *dumpPathLabel = new QLabel( backupGBox );
+	dumpPathLabel->setText( QString(i18n( "Path to '%1':" )).arg(sqliteBinary) );
+	dumpPathRequester = new KURLRequester( backupGBox );
+	dumpPathRequester->setFilter( sqliteBinary );
 
 	// Load & Save Current Settings
 	KConfig *config = kapp->config();
 	config->setGroup( "Server" );
-	fileEdit->setText( config->readEntry( "DBFile", locateLocal( "appdata", "krecipes.krecdb" ) ) );
+	fileRequester->setURL( config->readEntry( "DBFile", locateLocal( "appdata", "krecipes.krecdb" ) ) );
+	dumpPathRequester->setURL( config->readEntry( "SQLitePath", sqliteBinary ) );
 }
 
 void SQLiteServerPrefs::saveOptions( void )
 {
 	KConfig * config = kapp->config();
 	config->setGroup( "Server" );
-	config->writeEntry( "DBFile", fileEdit->text() );
-}
-
-void SQLiteServerPrefs::selectFile()
-{
-	KFileDialog dialog( QString::null, "*.*|All Files", this, "dialog", true );
-	if ( dialog.exec() == QDialog::Accepted ) {
-		fileEdit->setText( dialog.selectedFile() );
-	}
+	config->writeEntry( "DBFile", fileRequester->url() );
+	config->writeEntry( "SQLitePath", dumpPathRequester->url() );
 }
 
 
@@ -425,7 +486,7 @@ void NumbersPrefs::languageChange()
 	decimalRadioButton->setText( i18n( "Decimal" ) );
 }
 
-//=============Import Preferences Dialog================//
+//=============Import/Export Preferences Dialog================//
 ImportPrefs::ImportPrefs( QWidget *parent )
 		: QWidget( parent )
 {
@@ -438,16 +499,44 @@ ImportPrefs::ImportPrefs( QWidget *parent )
 
 	Form1Layout = new QVBoxLayout( this, 11, 6 );
 
-	overwriteCheckbox = new QCheckBox( i18n( "Overwrite recipes with same title" ), this );
+	QGroupBox *importGroup = new QGroupBox(2,Qt::Vertical,i18n("Import"), this);
+
+	overwriteCheckbox = new QCheckBox( i18n( "Overwrite recipes with same title" ), importGroup );
 	overwriteCheckbox->setChecked( overwrite );
 	overwriteCheckbox->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Minimum );
 
-	directImportCheckbox = new QCheckBox( i18n( "Ask which recipes to import" ), this );
+	directImportCheckbox = new QCheckBox( i18n( "Ask which recipes to import" ), importGroup );
 	directImportCheckbox->setChecked( !direct );
 	directImportCheckbox->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Minimum );
 
-	Form1Layout->addWidget( overwriteCheckbox );
-	Form1Layout->addWidget( directImportCheckbox );
+	Form1Layout->addWidget(importGroup);
+
+	QGroupBox *exportGroup = new QGroupBox(1,Qt::Vertical,i18n("Export"), this);
+
+	QHBox *clipboardHBox = new QHBox(exportGroup);
+	clipboardHBox->setSpacing(6);
+	QLabel *clipboardLabel = new QLabel(i18n("'Copy to Clipboard' format:"),clipboardHBox);
+	clipBoardFormatComboBox = new QComboBox( clipboardHBox );
+	clipBoardFormatComboBox->insertItem(QString("%3 (*.txt)").arg(i18n("Plain Text")));
+	clipBoardFormatComboBox->insertItem("Krecipes (*.kreml)");
+	clipBoardFormatComboBox->insertItem("Meal-Master (*.mmf)");
+	clipBoardFormatComboBox->insertItem("Rezkonv (*.rk)");
+	clipBoardFormatComboBox->insertItem("RecipeML (*.xml)");
+	//clipBoardFormatComboBox->insertItem("CookML (*.cml)");
+	clipboardHBox->setStretchFactor(clipBoardFormatComboBox,1);
+
+	config->setGroup( "Export" );
+	QString clipboardFormat = config->readEntry("ClipboardFormat");
+	if ( clipboardFormat == "*.kreml" )
+		clipBoardFormatComboBox->setCurrentItem(1);
+	else if ( clipboardFormat == "*.mmf" )
+		clipBoardFormatComboBox->setCurrentItem(2);
+	else if ( clipboardFormat == "*.xml" )
+		clipBoardFormatComboBox->setCurrentItem(3);
+	else
+		clipBoardFormatComboBox->setCurrentItem(0);
+
+	Form1Layout->addWidget(exportGroup);
 
 	Form1Layout->addItem( new QSpacerItem( 20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
 
@@ -467,6 +556,10 @@ void ImportPrefs::saveOptions()
 
 	config->writeEntry( "OverwriteExisting", overwriteCheckbox->isChecked() );
 	config->writeEntry( "DirectImport", !directImportCheckbox->isChecked() );
+
+	config->setGroup( "Export" );
+	QString ext = clipBoardFormatComboBox->currentText().mid(clipBoardFormatComboBox->currentText().find("(")+1,clipBoardFormatComboBox->currentText().length()-clipBoardFormatComboBox->currentText().find("(")-2);
+	config->writeEntry( "ClipboardFormat", ext );
 }
 
 
