@@ -375,9 +375,9 @@ ServerSetupPage::ServerSetupPage( QWidget *parent ) : QWidget( parent )
 
 	// Explanation text
 	serverSetupText = new QLabel( this );
-	serverSetupText->setText( i18n( "In this dialog you can adjust the database server settings." ) );
+	serverSetupText->setText( i18n( "In this dialog you can adjust the database server settings.<br><br><b>Warning: Passwords are stored in plain text and could potentially be compromised.  We recommend that you create a username and password combination solely for use by Krecipes.</b>" ) );
 	serverSetupText->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Minimum );
-	serverSetupText->setAlignment( int( QLabel::AlignTop | QLabel::AlignJustify ) );
+	serverSetupText->setAlignment( int( QLabel::AlignTop | QLabel::WordBreak ) );
 	layout->addWidget( serverSetupText, 1, 3 );
 
 	// Text spacer
@@ -474,8 +474,8 @@ ServerSetupPage::ServerSetupPage( QWidget *parent ) : QWidget( parent )
 	QSpacerItem* bottomSpacer = new QSpacerItem( 10, 10, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
 	layout->addItem( bottomSpacer, 8, 1 );
 
-	QSpacerItem* spacerRight = new QSpacerItem( 10, 10, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
-	layout->addItem( spacerRight, 1, 6 );
+	//QSpacerItem* spacerRight = new QSpacerItem( 10, 10, QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
+	//layout->addItem( spacerRight, 7, 4 );
 
 	// Signals & Slots
 	connect( remoteServerCheckBox, SIGNAL( toggled( bool ) ), serverSettingsGBox, SLOT( setEnabled( bool ) ) );
@@ -653,7 +653,7 @@ void SetupWizard::save( void )
 
 	config->setGroup( "Wizard" );
 	config->writeEntry( "SystemSetup", true );
-	config->writeEntry( "Version", "0.8" );
+	config->writeEntry( "Version", "0.9" );
 	kdDebug() << "Setting in kconfig the lines to disable wizard startup..." << sDBType << endl;
 }
 
@@ -676,6 +676,8 @@ void SetupWizard::getAdminInfo( bool &enabled, QString &adminUser, QString &admi
 void SetupWizard::getServerInfo( bool &isRemote, QString &host, QString &client, QString &dbName, QString &user, QString &pass )
 {
 	serverSetupPage->getServerInfo( isRemote, host, client, dbName, user, pass );
+	if ( dbTypeSetupPage->dbType() == SQLite )
+		dbName = sqliteSetupPage->dbFile();
 }
 
 DataInitializePage::DataInitializePage( QWidget *parent ) : QWidget( parent )
@@ -779,7 +781,7 @@ DBTypeSetupPage::DBTypeSetupPage( QWidget *parent ) : QWidget( parent )
 
 	liteCheckBox = new QRadioButton( i18n( "Simple Local File (SQLite)" ), bg, "liteCheckBox" );
 	mysqlCheckBox = new QRadioButton( i18n( "Local or Remote MySQL Database" ), bg, "liteCheckBox" );
-	psqlCheckBox = new QRadioButton( i18n( "Local or Remote PostgreSQL Database" ) + i18n(" (WARNING: **Beta**)"), bg, "psqlCheckBox" );
+	psqlCheckBox = new QRadioButton( i18n( "Local or Remote PostgreSQL Database" ), bg, "psqlCheckBox" );
 	bg->setButton( 0 ); // By default, SQLite
 
 #if (!HAVE_MYSQL)
