@@ -13,23 +13,46 @@
 
 #include <klineedit.h>
 
-#include "mixednumber.h"
+#include "datablocks/mixednumber.h"
 
-/** A KLineEdit widget extended to allow input of decimals and fractions.
+class QTimer;
+
+/** A KLineEdit widget extended to allow input of decimals and fractions or ranges of such.
   * Input is returned as a @ref MixedNumber class.
   * @author Jason Kivlighn
   */
 class FractionInput : public KLineEdit
 {
+Q_OBJECT
+
 public:
-	FractionInput( QWidget *parent = 0, const char *name = 0 );
+	FractionInput( QWidget *parent = 0, MixedNumber::Format = MixedNumber::MixedNumberFormat );
 	~FractionInput();
 
-	void setValue( double );
-	void setValue( const MixedNumber & );
+	void setAllowRange( bool b ){ m_allowRange = b; }
 
+	void setValue( double amount, double amount_offset );
+	void setValue( const MixedNumber &, double amount_offset );
+
+	void value( MixedNumber &amount, double &amount_offset ) const;
+	void value( double &amount, double &amount_offset ) const;
+	MixedNumber minValue() const;
+	MixedNumber maxValue() const;
 	MixedNumber value() const;
+
 	bool isInputValid() const;
+	bool isEmpty() const;
+
+public slots:
+	void validate();
+
+private slots:
+	void slotStartValidateTimer();
+
+private:
+	bool m_allowRange;
+	QTimer *m_validateTimer;
+	MixedNumber::Format m_format;
 };
 
 #endif //FRACTIONINPUT_H
